<?php
/**
 * ownCloud Firewall
 *
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Firewall;

use OCP\Files\IMimeTypeDetector;
use OCP\IGroupManager;
use OCP\IRequest;
use OCP\IUser;
use OCP\IUserSession;

class Context {
	/**
	 * DO NOT CHANGE without updating the config and docs
	 */
	const REQUEST_TYPE_OTHER = 'other';
	const REQUEST_TYPE_PUBLIC = 'public';
	const REQUEST_TYPE_WEBDAV = 'webdav';
	const REQUEST_TYPE_UPLOAD = 'upload';
	const REQUEST_TYPE_FILES_DROP = 'files_drop';

	/**
	 * DO NOT CHANGE without updating the config and docs
	 */
	const SYNC_CLIENT_ANDROID = 'android';
	const SYNC_CLIENT_ANDROID_BRANDED = 'android_branded';
	const SYNC_CLIENT_DESKTOP = 'desktop';
	const SYNC_CLIENT_DESKTOP_BRANDED = 'desktop_branded';
	const SYNC_CLIENT_IOS = 'ios';
	const SYNC_CLIENT_IOS_BRANDED = 'ios_branded';
	const SYNC_CLIENT_OTHER = 'other';
	const SYNC_CLIENT_BRANDED = 'branded';
	const SYNC_CLIENT_NON_BRANDED = 'non_branded';

	/** @var IRequest */
	protected $request;

	/** @var IGroupManager */
	protected $groupManager;

	/** @var IUserSession */
	protected $userSession;

	/** @var IMimeTypeDetector */
	protected $mimeTypeDetector;

	/** @var Config */
	protected $config;

	/**
	 * @param IRequest $request
	 * @param IGroupManager $groupManager
	 * @param IUserSession $userSession
	 * @param IMimeTypeDetector $mimeTypeDetector
	 * @param Config $config
	 */
	public function __construct(IRequest $request, IGroupManager $groupManager,  IUserSession $userSession, IMimeTypeDetector $mimeTypeDetector, Config $config) {
		$this->request = $request;
		$this->groupManager = $groupManager;
		$this->userSession = $userSession;
		$this->mimeTypeDetector = $mimeTypeDetector;
		$this->config = $config;
	}

	/*
	 * Request related methods
	 */

	/**
	 * @return string
	 */
	public function getRequestFullUrl() {
		return $this->request->getServerProtocol() . '://'
			. $this->request->getServerHost()
			. $this->request->getRequestUri();
	}

	/**
	 * @return string
	 */
	public function getRequestUri() {
		return $this->request->getRequestUri();
	}

	/**
	 * @param bool $verbose If true upload can also be returned
	 * @return string webdav, public or other
	 */
	public function getRequestType($verbose = false) {
		if (
			// public.php
			$this->isScriptName('public.php') ||
			// share links via index.php/s/<token>
			($this->isScriptName('index.php') && substr($this->request->getPathInfo(), 0, strlen('/s/')) === '/s/')
		) {
			return self::REQUEST_TYPE_PUBLIC;
		}

		if ($this->isScriptName('remote.php')
			&& ($this->request->getPathInfo() === '/webdav'
				|| substr($this->request->getPathInfo(), 0, strlen('/webdav/')) === '/webdav/')) {
			return self::REQUEST_TYPE_WEBDAV;
		}

		if ($verbose) {
			if ($this->isScriptName('index.php') && $this->request->getPathInfo() === '/apps/files/ajax/upload.php') {
				return self::REQUEST_TYPE_UPLOAD;
			}

			// Files drop via index.php/apps/files_drop/<token>
			if ($this->isScriptName('index.php') && substr($this->request->getPathInfo(), 0, strlen('/apps/files_drop/')) === '/apps/files_drop/') {
				return self::REQUEST_TYPE_FILES_DROP;
			}
		}

		return self::REQUEST_TYPE_OTHER;
	}

	/**
	 * @param string $toFind
	 * @return bool
	 */
	protected function isScriptName($toFind) {
		return substr($this->request->getScriptName(), 0 - strlen('/' . $toFind)) === '/' . $toFind;
	}

	/**
	 * @return int
	 */
	public function getRequestTime() {
		return time();
	}

	/**
	 * @return int
	 */
	public function getUploadSize() {
		$checkSize = false;
		if ($this->getRequestType() === self::REQUEST_TYPE_WEBDAV) {
			if ($this->request->getMethod() === 'PUT') {
				$checkSize = true;
			}
		} else if ($this->getRequestType(true) === self::REQUEST_TYPE_FILES_DROP) {
			if ($this->request->getMethod() === 'POST') {
				$file = $this->request->getUploadedFile('file');
				$checkSize = !empty($file);
			}
		} else {
			if (in_array($this->request->getMethod(), ['POST', 'PUT'])) {
				$files = $this->request->getUploadedFile('files');
				$checkSize = !empty($files);
			}
		}

		if ($checkSize) {
			$uploadSize = $this->request->getHeader('OC_TOTAL_LENGTH');
			if ($uploadSize !== null) {
				return $uploadSize;
			}

			$uploadSize = $this->request->getHeader('CONTENT_LENGTH');
			if ($uploadSize !== null) {
				return $uploadSize;
			}
		}

		return 0;
	}

	/**
	 * @return string|null
	 */
	public function getUploadType() {
		if ($this->getRequestType() === self::REQUEST_TYPE_WEBDAV) {
			if ($this->request->getMethod() === 'PUT') {
				return $this->mimeTypeDetector->detectPath($this->request->getPathInfo());
			}
		} else if ($this->getRequestType(true) === self::REQUEST_TYPE_FILES_DROP) {
			if ($this->request->getMethod() === 'POST') {
				$file = $this->request->getUploadedFile('file');
				if (isset($file['type']) && is_string($file['type'])) {
					return $file['type'];
				}
			}
		} else {
			if (in_array($this->request->getMethod(), ['POST', 'PUT'])) {
				$files = $this->request->getUploadedFile('files');
				if (isset($files['type'][0])) {
					return $files['type'][0];
				}
			}
		}

		return null;
	}

	/*
	 * User related methods
	 */

	/**
	 * @return string[]
	 */
	public function getUserGroups() {
		$user = $this->userSession->getUser();

		if ($user instanceof IUser) {
			return $this->groupManager->getUserGroupIds($user);
		} else {
			return [];
		}
	}

	/*
	 * Client related methods
	 */

	/**
	 * @return string
	 */
	public function getRemoteAddress() {
		return $this->request->getRemoteAddress();
	}

	/**
	 * @return string
	 */
	public function getRemoteServerAddress() {
		$header = $this->request->getHeader('SERVER_ADDR');
		return $header !== null ? $header : '';
	}

	/**
	 * @return string
	 */
	public function getUserAgent() {
		return $this->request->getHeader('User-Agent');
	}

	/**
	 * @return string
	 */
	public function getClientDevice() {
		$userAgent = strtolower($this->getUserAgent());

		$brandedClients = $this->config->getBrandedClients();
		if (!empty($brandedClients) && is_array($brandedClients)) {
			$device = $this->findDevice($userAgent, $brandedClients);

			if ($device !== self::SYNC_CLIENT_OTHER) {
				// Only send the device if we found one, otherwise try the
				// default clients
				return $device;
			}
		}

		$defaultClients = [
			// Current user agents
			// Mozilla/5.0 (<os>) mirall/<version>
			'*) mirall/*' => self::SYNC_CLIENT_DESKTOP,
			'Mozilla/5.0 (Android) ownCloud-android/*' => self::SYNC_CLIENT_ANDROID,
			'Mozilla/5.0 (iOS) ownCloud-iOS/*' => self::SYNC_CLIENT_IOS,

			// Older user agents
			'*mirall*' => self::SYNC_CLIENT_DESKTOP,
			'*csync*' => self::SYNC_CLIENT_DESKTOP,

			'*iOS-ownCloud' => self::SYNC_CLIENT_IOS,
			'*ownCloud iOS Client*' => self::SYNC_CLIENT_IOS,

			'*Android-ownCloud*' => self::SYNC_CLIENT_ANDROID,
			'Mozilla/5.0 (Android) ownCloud*' => self::SYNC_CLIENT_ANDROID,
		];

		return $this->findDevice($userAgent, $defaultClients);
	}

	protected function findDevice($userAgent, $devices) {

		foreach ($devices as $clientAgent => $device) {
			if ($this->userAgentMatchesDevice($userAgent, strtolower($clientAgent)) !== false) {
				return $device;
			}
		}

		return self::SYNC_CLIENT_OTHER;
	}

	/**
	 * @param string $userAgent
	 * @param string $device
	 * @return bool
	 */
	protected function userAgentMatchesDevice($userAgent, $device) {
		if (strlen($device) <= 3) {
			return false;
		}

		$device = strtolower($device);

		// No wildcard means exact match
		if (substr($device, 0, 1) !== '*' && substr($device, -1) !== '*') {
			return $userAgent === $device;
		}

		// Trailing wildcard only
		if (substr($device, 0, 1) !== '*') {
			$beginning = substr($device, 0, -1);
			return substr($userAgent, 0, strlen($beginning)) === $beginning;
		}

		// Leading wildcard only
		if (substr($device, -1) !== '*') {
			$ending = substr($device, 1);
			return substr($userAgent, 0 - strlen($ending)) === $ending;
		}

		// Double wildcard, find anywhere
		return strpos($userAgent, substr($device, 1, -1)) !== false;
	}
}
