<?php
/**
 * ownCloud Firewall
 *
 * @author Clark Tomlinson
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Firewall;

use OCP\AppFramework\Http;
use OCP\AppFramework\Http\Response;
use OCP\AppFramework\Http\JSONResponse;
use OCP\AppFramework\Http\TemplateResponse;
use OCP\IGroupManager;
use OCP\IRequest;
use OCP\IURLGenerator;
use OCP\IUser;
use OCP\IUserSession;
use OCP\L10N\IFactory;
use OCP\SabrePluginEvent;
use OCP\Util;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

class Firewall {

	/** @var Config */
	protected $config;

	/** @var Context */
	protected $context;

	/** @var Rules */
	protected $rules;

	/** @var IUserSession */
	protected $session;

	/** @var IGroupManager */
	protected $groupManager;

	/** @var IRequest */
	protected $request;

	/** @var IURLGenerator */
	protected $urlGenerator;

	/**
	 * @param Config $config
	 * @param Context $context
	 * @param Rules $rules
	 * @param Logger $logger
	 * @param IUserSession $session
	 * @param EventDispatcherInterface $dispatcher
	 * @param IGroupManager $groupManager
	 * @param IRequest $request
	 * @param IFactory $languageFactory
	 * @param IURLGenerator $urlGenerator
	 */
	public function __construct(Config $config,
								Context $context,
								Rules $rules,
								Logger $logger,
								IUserSession $session,
								EventDispatcherInterface $dispatcher,
								IGroupManager $groupManager,
								IRequest $request,
								IFactory $languageFactory,
								IURLGenerator $urlGenerator) {
		$this->config = $config;
		$this->context = $context;
		$this->rules = $rules;
		$this->session = $session;
		$this->dispatcher = $dispatcher;
		$this->groupManager = $groupManager;
		$this->logger = $logger;
		$this->request = $request;
		$this->languageFactory = $languageFactory;
		$this->urlGenerator = $urlGenerator;
	}

	/**
	 * Boot the firewall
	 *
	 * Either checks the request, or registers to an event in case of webdav
	 */
	public function boot() {
		$rules = $this->config->getRules(true);

		// Check the request against rules if we have some
		if (!empty($rules)) {
			if ($this->context->getRequestType(true) === Context::REQUEST_TYPE_OTHER) {
				$this->checkWebUiRequest($rules);

			} else if ($this->context->getRequestType() === Context::REQUEST_TYPE_WEBDAV) {
				// Register for the webdav plugin because auth is not set up yet
				$this->dispatcher->addListener(
					'OC\Connector\Sabre::beforeMethod',
					function (SabrePluginEvent $event) use ($rules) {
						$this->checkRequest($rules, $event);
					}
				);

			} else {
				$this->checkRequest($rules);
			}
		}
	}

	/**
	 * Check if the current request is allowed or not
	 * NOTE: Skips the check for guests and admins, to allow them logging in any time.
	 *
	 * @param array $rules
	 * @return bool True if the request should be allowed
	 * @throws \Exception when the request is blocked
	 */
	public function checkWebUiRequest(array $rules) {

		// We don't want to block requests to login or admin users
		$user = $this->session->getUser();

		$logout = $this->request->getParam('logout', 'false');
		$isAdmin = false;
		if ($user instanceof IUser) {
			$isAdmin = $this->groupManager->isAdmin($user->getUID());
		}

		if (!($user instanceof IUser) || $isAdmin || $logout === 'true') {
			$this->logger->onSkip($isAdmin);
			return true;
		}

		return $this->checkRequest($rules);
	}

	/**
	 * Check if the current request is allowed or not
	 *
	 * @param array $rules
	 * @param SabrePluginEvent $event
	 * @return bool True if the request should be allowed
	 * @throws \Exception when the request is blocked
	 */
	public function checkRequest(array $rules, SabrePluginEvent $event = null) {
		if ($this->rules->check($rules)) {
			$this->logger->onPass();
			return true;
		} else {
			$this->logger->onFail($this->rules->getOffender());
			return $this->failRequest($event);
		}
	}

	/**
	 * Runs when a rule fail
	 *
	 * @param SabrePluginEvent $event
	 * @return bool
	 */
	protected function failRequest(SabrePluginEvent $event = null) {
		$l = $this->languageFactory->get('firewall');

		if ($event instanceof SabrePluginEvent) {
			$event->setMessage((string) $l->t('Access to this resource has been forbidden by a file firewall rule.'))
				->setStatusCode(Http::STATUS_FORBIDDEN)
				->stopPropagation();
			return false;
		} else {
			if ($this->context->getRequestType(true) === Context::REQUEST_TYPE_UPLOAD) {
				$response = new JSONResponse([
					'data' => [
						'message' => (string) $l->t('Access to this resource has been forbidden by a file firewall rule.'),
					],
					'status' => 'error',
				]);
			} else if ($this->context->getRequestType(true) === Context::REQUEST_TYPE_FILES_DROP) {
				$response = new JSONResponse([
					'message' => (string) $l->t('Access to this resource has been forbidden by a file firewall rule.'),
				]);
			} else {
				// TODO use a hint exception once index.php and ocs allow setting the status code
//				throw new \OC\HintException(
//					$l->t('Access to this resource has been forbidden by a file firewall rule.'),
//					$l->t('If you feel this is an error, please contact your administrator'),
//					403
//				);

				$response = new TemplateResponse('firewall', 'failure', [
					'l10n' => $l,
					'logoutUrl' => $this->urlGenerator->linkTo('', 'index.php', [
						'logout'	=> 'true',
						'requesttoken'	=> Util::callRegister(),
					])
				], 'guest');
			}


			$this->sendResponse($response);
			return false;
		}
	}

	/**
	 * @param Response $response
	 * @codeCoverageIgnore
	 */
	protected function sendResponse(Response $response) {
		http_response_code(Http::STATUS_FORBIDDEN);
		echo $response->render();
		exit;
	}
}
