<?php
/**
 * ownCloud Firewall
 *
 * @author Clark Tomlinson
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Firewall;


class Rules {

	/** @var string */
	protected $offender;

	/** @var RuleFactory */
	protected $ruleFactory;

	/** @var Context */
	protected $context;

	/**
	 * Constructor to build a set of rules to evaluate against
	 *
	 * @param RuleFactory $ruleFactory
	 * @param Context $context
	 */
	public function __construct(RuleFactory $ruleFactory, Context $context) {
		$this->ruleFactory = $ruleFactory;
		$this->context = $context;
	}

	/**
	 * Evaluate a condition rule to true or false based on the request criteria
	 *
	 * @param array $rule The condition rule to evaluate
	 * @return bool True if the criteria matches the rule condition, false if not
	 */
	protected function evalCondition(array $rule) {
		if (!isset($rule['id'])) {
			return false;
		}

		try {
			$ruleInstance = $this->ruleFactory->getRuleInstance($rule['id']);
		} catch (\InvalidArgumentException $e) {
			// Invalid rule, should log this
			return false;
		}

		return $ruleInstance->doCheck($rule['operator'], $rule['value']);
	}

	/**
	 * Validate whether config is a correct set of rules
	 *
	 * @param string $config JSON encoded set of rules
	 * @return string Returns a cleaned version of the rule set (removes empty rules)
	 * @throws \OutOfBoundsException when the rule set is invalid
	 */
	public function validateRules($config) {
		if (!is_string($config)) {
			throw new \OutOfBoundsException('The firewall rule config is not a valid json array');
		}

		$config = json_decode($config, true);

		if (!is_array($config)) {
			throw new \OutOfBoundsException('The firewall rule config is not a valid json array');
		}

		foreach ($config as $i => $ruleSet) {
			if (empty($ruleSet)) {
				// This happens when you delete the last rule and then save
				unset($config[$i]);
				continue;
			}

			if (!is_array($ruleSet) || sizeof($ruleSet) !== 2 || !isset($ruleSet['name']) || !is_string($ruleSet['name']) || !isset($ruleSet['rules']) || !is_array($ruleSet['rules'])) {
				throw new \OutOfBoundsException('The firewall rule #' . $i . ' is invalid');
			}

			foreach ($ruleSet['rules'] as $j => $rule) {
				$this->validateRule($rule, $ruleSet['name'] . ' - Condition #' . $j);
			}
		}

		return json_encode($config);
	}

	/**
	 * Validate whether the operator and compare value are valid for this rule
	 *
	 * @param array $rule The condition rule to evaluate
	 * @param string $ruleIdentifier
	 * @return bool
	 * @throws \OutOfBoundsException when the rule is invalid
	 */
	protected function validateRule($rule, $ruleIdentifier) {
		if (!isset($rule['id'])) {
			throw new \OutOfBoundsException('Missing firewall rule id in "' . $ruleIdentifier . '"');
		}
		if (isset($rule['rules']) || !isset($rule['operator']) || !is_string($rule['operator']) || !isset($rule['value'])) {
			throw new \OutOfBoundsException('Invalid firewall rule "' . $ruleIdentifier . '"');
		}

		try {
			$ruleInstance = $this->ruleFactory->getRuleInstance($rule['id']);
		} catch (\InvalidArgumentException $e) {
			throw new \OutOfBoundsException($e->getMessage() . ' for rule "' . $ruleIdentifier . '"', $e->getCode(), $e);
		}

		$ruleInstance->validateOperator($rule['operator'], $rule['id'] . ' - ' . $ruleIdentifier);
		$ruleInstance->validateRuleValue($rule['value'], $rule['id'] . ' - ' . $ruleIdentifier);

		return true;
	}

	/**
	 * @return string
	 */
	public function getOffender() {
		return $this->offender;
	}

	/**
	 * Check if a request is allowed
	 *
	 * @param array $rules
	 * @return bool True if the request should be accepted, false otherwise
	 */
	public function check($rules) {
		// If our rules are empty user has not defined any thus allow the request
		if (empty($rules)) {
			return true;
		}

		foreach ($rules as $key => $rule) {
			if (empty($rule['rules'])) {
				continue;
			}

			foreach ($rule['rules'] as $conditionNum => $condition) {
				if (!$this->evalCondition($condition)) {
					// The rule condition did not match, so we can skip the
					// remaining conditions of this rule and continue with the
					// next rule.
					continue 2;
				}
			}

			// All conditions matched, so the rule matches as well and the
			// request should be blocked.
			$this->offender = $rule['name'];
			return false;
		}

		return true;
	}
}
