<?php
/**
 * ownCloud Firewall
 *
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Firewall\Tests\Controller;


use OCA\Firewall\Controller\RulesController;
use OCP\AppFramework\Http;
use OCP\AppFramework\Http\DataResponse;
use Test\TestCase;

class RulesControllerTest extends TestCase {

	/** @var \OCA\Firewall\Config|\PHPUnit_Framework_MockObject_MockObject */
	protected $config;

	/** @var \OCA\Firewall\Rules|\PHPUnit_Framework_MockObject_MockObject */
	protected $rules;

	/** @var \OCA\Firewall\RuleFactory|\PHPUnit_Framework_MockObject_MockObject */
	protected $ruleFactory;

	/** @var \OCP\IRequest|\PHPUnit_Framework_MockObject_MockObject */
	protected $request;

	/** @var \OCP\IUserSession|\PHPUnit_Framework_MockObject_MockObject */
	protected $userSession;

	/** @var RulesController */
	protected $controller;


	protected function setUp() {
		parent::setUp();

		$this->config = $this->getMockBuilder('OCA\Firewall\Config')
			->disableOriginalConstructor()
			->getMock();

		$this->rules = $this->getMockBuilder('OCA\Firewall\Rules')
			->disableOriginalConstructor()
			->getMock();

		$this->ruleFactory = $this->getMockBuilder('OCA\Firewall\RuleFactory')
			->disableOriginalConstructor()
			->getMock();

		$this->request = $this->getMockBuilder('OCP\IRequest')
			->disableOriginalConstructor()
			->getMock();

		$this->userSession = $this->getMockBuilder('OCP\IUserSession')
			->disableOriginalConstructor()
			->getMock();

		$this->controller = new RulesController(
			'firewall',
			$this->request,
			$this->rules,
			$this->ruleFactory,
			$this->config,
			$this->userSession
		);
	}

	public function dataSave() {
		return [
			['json-encoded-rules1', false, false, Http::STATUS_OK, []],
			['json-encoded-rules2', true, false, Http::STATUS_NOT_ACCEPTABLE, ['message' => 'Validation error']],
			['json-encoded-rules3', true, true, Http::STATUS_NOT_ACCEPTABLE, ['message' => 'Validation error']],
			['json-encoded-rules4', false, true, Http::STATUS_INTERNAL_SERVER_ERROR, ['message' => 'Config error']],
		];
	}

	/**
	 * @dataProvider dataSave
	 *
	 * @param string $rules
	 * @param bool $validationThrows
	 * @param bool $configSaveThrows
	 * @param int $statusCode
	 * @param array $expectedData
	 */
	public function testSave($rules, $validationThrows, $configSaveThrows, $statusCode, array $expectedData) {
		if ($validationThrows) {
			$this->rules->expects($this->once())
				->method('validateRules')
				->with($rules)
				->willThrowException(new \OutOfBoundsException('Validation error'));
		} else {
			$this->rules->expects($this->once())
				->method('validateRules')
				->with($rules)
				->willReturn($rules . 'modifiedByValidateRules');
		}

		if ($configSaveThrows) {
			$this->config->expects($validationThrows ? $this->never() : $this->once())
				->method('setRules')
				->with($rules . 'modifiedByValidateRules')
				->willThrowException(new \Exception('Config error'));
		} else {
			$this->config->expects($validationThrows ? $this->never() : $this->once())
				->method('setRules')
				->with($rules . 'modifiedByValidateRules');
		}

		/** @var DataResponse $response */
		$response = $this->controller->save($rules);
		$this->assertInstanceOf('OCP\AppFramework\Http\DataResponse', $response);
		$this->assertSame($statusCode, $response->getStatus());
		$this->assertSame($expectedData, $response->getData());
	}

	public function dataDebug() {
		return [
			[0, false, Http::STATUS_OK, []],
			[1, true, Http::STATUS_INTERNAL_SERVER_ERROR, ['message' => 'Config error']],
		];
	}

	/**
	 * @dataProvider dataDebug
	 *
	 * @param int $level
	 * @param bool $configSaveThrows
	 * @param int $statusCode
	 * @param array $expectedData
	 */
	public function testDebug($level, $configSaveThrows, $statusCode, array $expectedData) {
		if ($configSaveThrows) {
			$this->config->expects($this->once())
				->method('setDebug')
				->with($level)
				->willThrowException(new \Exception('Config error'));
		} else {
			$this->config->expects($this->once())
				->method('setDebug')
				->with($level);
		}

		/** @var DataResponse $response */
		$response = $this->controller->debug($level);
		$this->assertInstanceOf('OCP\AppFramework\Http\DataResponse', $response);
		$this->assertSame($statusCode, $response->getStatus());
		$this->assertSame($expectedData, $response->getData());
	}

	public function dataGetUIData() {
		return [
			['rules1', 1, false],
			['rules2', 2, 'Error'],
		];
	}

	/**
	 * @dataProvider dataGetUIData
	 *
	 * @param string $rules
	 * @param int $level
	 * @param string $validationError
	 */
	public function testGetUIData($rules, $level, $validationError) {
		$this->config->expects($this->once())
			->method('getRules')
			->willReturn($rules);
		$this->config->expects($this->once())
			->method('getDebugLevel')
			->willReturn($level);

		if ($validationError !== false) {
			$this->rules->expects($this->once())
				->method('validateRules')
				->with($rules)
				->willThrowException(new \OutOfBoundsException($validationError));
		} else {
			$this->rules->expects($this->once())
				->method('validateRules')
				->with($rules);
		}

		$this->ruleFactory->expects($this->once())
			->method('getRules')
			->willReturn([]);

		/** @var DataResponse $response */
		$response = $this->controller->getUIData();
		$this->assertInstanceOf('OCP\AppFramework\Http\DataResponse', $response);
		$this->assertSame(Http::STATUS_OK, $response->getStatus());
		$this->assertSame([
			'rules' => $rules,
			'validationError' => $validationError,
			'debugLevel' => $level,
			'ruleValues' => [],
		], $response->getData());
	}


	/**
	 * @param array $validValues
	 * @return \OCA\Firewall\Rules\Contracts\Rule|\PHPUnit_Framework_MockObject_MockObject
	 */
	protected function getRuleMock($validValues) {
		$mock = $this->getMockBuilder('OCA\Firewall\Rules\Contracts\Rule')
			->disableOriginalConstructor()
			->setMethods([
				'doCheck',
				'validateRuleValue',
				'getValidOperators',
				'getValidValues',
			])
			->getMock();
		$mock->expects($this->any())
			->method('getValidValues')
			->willReturn($validValues);
		return $mock;
	}

	public function dataGetUIDataRuleValues() {
		return [
			[
				['test1', 'test2'],
				[
					['test1', $this->getRuleMock(['one' => '1'])],
					['test2', $this->getRuleMock(['two' => '2'])],
				],
				[
					'test1' => ['one' => '1'],
					'test2' => ['two' => '2'],
				],
			],
		];
	}

	/**
	 * @dataProvider dataGetUIDataRuleValues
	 *
	 * @param array $ruleIds
	 * @param array $factoryMap
	 * @param array $expected
	 */
	public function testGetUIDataRuleValues($ruleIds, $factoryMap, $expected) {
		$this->config->expects($this->once())
			->method('getRules');
		$this->config->expects($this->once())
			->method('getDebugLevel');
		$this->rules->expects($this->once())
				->method('validateRules');

		$this->ruleFactory->expects($this->once())
			->method('getRules')
			->willReturn($ruleIds);
		$this->ruleFactory->expects($this->any())
			->method('getRuleInstance')
			->willReturnMap($factoryMap);

		/** @var DataResponse $response */
		$response = $this->controller->getUIData();
		$this->assertInstanceOf('OCP\AppFramework\Http\DataResponse', $response);
		$this->assertSame(Http::STATUS_OK, $response->getStatus());
		$this->assertSame([
			'rules' => null,
			'validationError' => false,
			'debugLevel' => null,
			'ruleValues' => $expected,
		], $response->getData());
	}
}
