<?php
/**
 * @author Clark Tomlinson  <clark@owncloud.com>
 *
 * @copyright Copyright (c) 2015, ownCloud, Inc.
 * @license AGPL-3.0
 */


namespace OCA\Firewall\Tests\Lib;


use OCA\Firewall\Context;
use OCA\Firewall\Firewall;
use OCP\AppFramework\Http;
use OCP\SabrePluginEvent;
use Test\TestCase;

class FirewallTest extends TestCase {

	/** @var Firewall */
	private $instance;

	/** @var \OCA\Firewall\Config|\PHPUnit_Framework_MockObject_MockObject */
	protected $config;
	/** @var \OCA\Firewall\Context|\PHPUnit_Framework_MockObject_MockObject */
	protected $context;
	/** @var \OCA\Firewall\Rules|\PHPUnit_Framework_MockObject_MockObject */
	private $rulesMock;
	/** @var \OCA\Firewall\Logger|\PHPUnit_Framework_MockObject_MockObject */
	private $logger;

	/** @var \OCP\IUserSession|\PHPUnit_Framework_MockObject_MockObject */
	private $userSessionMock;
	/** @var \Symfony\Component\EventDispatcher\EventDispatcherInterface|\PHPUnit_Framework_MockObject_MockObject */
	protected $dispatcher;
	/** @var \OCP\IRequest|\PHPUnit_Framework_MockObject_MockObject */
	private $requestMock;
	/** @var \OCP\IGroupManager|\PHPUnit_Framework_MockObject_MockObject */
	private $groupManager;
	/** @var \OCP\L10N\IFactory|\PHPUnit_Framework_MockObject_MockObject */
	private $languageFactory;
	/** @var \OCP\IURLGenerator|\PHPUnit_Framework_MockObject_MockObject */
	protected $urlGenerator;

	public function testBootNoRules() {
		$instance = $this->getInstance(['checkRequest']);

		$this->config->expects($this->once())
			->method('getRules')
			->with(true)
			->willReturn([]);

		$this->context->expects($this->never())
			->method('getRequestType');
		$this->dispatcher->expects($this->never())
			->method('addListener');
		$instance->expects($this->never())
			->method('checkRequest');
		$instance->expects($this->never())
			->method('checkWebUiRequest');

		$instance->boot();
	}

	public function dataBootNotWebDAV() {
		return [
			[Context::REQUEST_TYPE_OTHER, 'checkWebUiRequest'],
			[Context::REQUEST_TYPE_PUBLIC, 'checkRequest'],
		];
	}

	/**
	 * @dataProvider dataBootNotWebDAV
	 *
	 * @param string $requestType
	 * @param string $checkMethod
	 */
	public function testBootNotWebDAV($requestType, $checkMethod) {
		$rules = ['rules'];
		$instance = $this->getInstance([
			'checkRequest',
			'checkWebUiRequest',
		]);

		$this->config->expects($this->once())
			->method('getRules')
			->with(true)
			->willReturn($rules);

		$this->context->expects($this->atLeastOnce())
			->method('getRequestType')
			->willReturn($requestType);
		$this->dispatcher->expects($this->never())
			->method('addListener');
		$instance->expects($this->once())
			->method($checkMethod)
			->with($rules);

		$instance->boot();
	}

	public function dataBootWebDAV() {
		return [
			[true],
			[false],
		];
	}

	/**
	 * @dataProvider dataBootWebDAV
	 * @param bool $executeListener
	 */
	public function testBootWebDAV($executeListener) {
		$event = new SabrePluginEvent();

		$rules = ['rules'];
		$instance = $this->getInstance(['checkRequest'], $executeListener);

		$this->config->expects($this->once())
			->method('getRules')
			->with(true)
			->willReturn($rules);

		$this->context->expects($this->atLeastOnce())
			->method('getRequestType')
			->willReturn(Context::REQUEST_TYPE_WEBDAV);

		if ($executeListener) {
			$instance->expects($this->once())
				->method('checkRequest')
				->with($rules, $this->anything());
		} else {
			$this->dispatcher->expects($this->once())
				->method('addListener')
				->with('OC\Connector\Sabre::beforeMethod', $this->anything());
			$instance->expects($this->never())
				->method('checkRequest');
		}

		$instance->boot();

		if ($executeListener) {
			\OC::$server->getEventDispatcher()->dispatch('OC\Connector\Sabre::beforeMethod', $event);
		}
	}

	public function dataCheckWebUiRequest() {
		return [
			['admin', false, true, 'onSkip', null, true],
			['logout', true, false, 'onSkip', null, true],
			['adminlogout', true, true, 'onSkip', null, true],

			// user logging in
			[null, false, null, 'onSkip', null, true],

			['user', false, false, null, true, true],
			['user', false, false, null, false, false],
		];
	}

	/**
	 * @dataProvider dataCheckWebUiRequest
	 *
	 * @param mixed $uid
	 * @param bool $isLoggingOut
	 * @param bool $isAdmin
	 * @param string $loggerMethod
	 * @param bool $expected
	 */
	public function testCheckWebUiRequest($uid, $isLoggingOut, $isAdmin, $loggerMethod, $checkRequest, $expected) {
		$this->userSessionMock->expects($this->once())
			->method('getUser')
			->willReturn($this->getUser($uid));

		$this->groupManager->expects(($isAdmin === null) ? $this->never() : $this->once())
			->method('isAdmin')
			->willReturn($isAdmin);

		$this->requestMock->expects($this->once())
			->method('getParam')
			->with('logout', 'false')
			->willReturn($isLoggingOut ? 'true' : 'false');

		if ($loggerMethod !== null) {
			$this->logger->expects($this->once())
				->method($loggerMethod)
				->with($isAdmin);
		}

		$instance = $this->getInstance([
			'sendResponse',
			'checkRequest',
		]);

		if ($checkRequest === null) {
			$instance->expects($this->never())
				->method('checkRequest');
		} else {
			$instance->expects($this->once())
				->method('checkRequest')
				->willReturn($checkRequest);
		}

		$this->assertSame($expected, $instance->checkWebUiRequest([]));
	}

	public function dataCheckRequest() {
		return [
			[true, null, 'onPass', true],
			[false, 'Failing rule1', 'onFail', false],
			[false, 'Failing rule2', 'onFail', false],
		];
	}

	/**
	 * @dataProvider dataCheckRequest
	 *
	 * @param bool $checkPasses
	 * @param string $offendingRule
	 * @param string $loggerMethod
	 * @param bool $expected
	 */
	public function testCheckRequest($checkPasses, $offendingRule, $loggerMethod, $expected) {

		$this->rulesMock->expects(($checkPasses === null) ? $this->never() : $this->once())
			->method('check')
			->willReturn($checkPasses);
		$this->rulesMock->expects(($checkPasses !== false) ? $this->never() : $this->once())
			->method('getOffender')
			->willReturn($offendingRule);

		if ($offendingRule === null) {
			$this->logger->expects($this->once())
				->method($loggerMethod);
		} else {
			$this->logger->expects($this->once())
				->method($loggerMethod)
				->with($offendingRule);
		}

		$instance = $this->getInstance([
			'sendResponse',
		]);

		$this->assertSame($expected, $instance->checkRequest([]));
	}

	protected function getUser($uid) {
		if ($uid === null) {
			return null;
		}

		$user = $this->getMockBuilder('OCP\IUser')
			->disableOriginalConstructor()
			->getMock();
		$user->expects($this->any())
			->method('getUID')
			->willReturn($uid);
		return $user;
	}

	public function dataFail() {
		return [
			[Context::REQUEST_TYPE_WEBDAV],
			[Context::REQUEST_TYPE_OTHER],
			[Context::REQUEST_TYPE_UPLOAD],
			[Context::REQUEST_TYPE_FILES_DROP],
		];
	}

	/**
	 * @dataProvider dataFail
	 *
	 * @param string $requestType
	 */
	public function testFail($requestType) {
		$instance = $this->getInstance([
			'sendResponse',
		]);

		$event = null;
		if ($requestType === Context::REQUEST_TYPE_WEBDAV) {
			$event = $this->getMockBuilder('OCP\SabrePluginEvent')
				->disableOriginalConstructor()
				->getMock();

			$event->expects($this->once())
				->method('setMessage')
				->willReturnSelf();
			$event->expects($this->once())
				->method('setStatusCode')
				->willReturnSelf();
			$event->expects($this->once())
				->method('stopPropagation');
		} else {
			if ($requestType === Context::REQUEST_TYPE_UPLOAD) {
				$this->context->expects($this->once())
					->method('getRequestType')
					->with(true)
					->willReturn($requestType);
			} else {
				$this->context->expects($this->exactly(2))
					->method('getRequestType')
					->with(true)
					->willReturn($requestType);
			}

			$instance->expects($this->once())
				->method('sendResponse')
				->willReturn(null);
		}

		$this->invokePrivate($instance, 'failRequest' , [$event]);
	}

	protected function setUp() {

		$this->config = $this->getMockBuilder('OCA\Firewall\Config')
			->disableOriginalConstructor()
			->getMock();

		$this->context = $this->getMockBuilder('OCA\Firewall\Context')
			->disableOriginalConstructor()
			->getMock();

		$this->rulesMock = $this->getMockBuilder('OCA\Firewall\Rules')
			->disableOriginalConstructor()
			->getMock();

		$this->userSessionMock = $this->getMockBuilder('OCP\IUserSession')
			->disableOriginalConstructor()
			->getMock();

		$this->groupManager = $this->getMockBuilder('OCP\IGroupManager')
			->disableOriginalConstructor()
			->getMock();

		$this->requestMock = $this->getMockBuilder('OCP\IRequest')
			->disableOriginalConstructor()
			->getMock();

		$this->logger = $this->getMockBuilder('OCA\Firewall\Logger')
			->disableOriginalConstructor()
			->getMock();

		$this->urlGenerator = $this->getMockBuilder('OCP\IURLGenerator')
			->disableOriginalConstructor()
			->getMock();

		$this->languageFactory = $this->getMockBuilder('OCP\L10N\IFactory')
			->disableOriginalConstructor()
			->getMock();

		$this->languageFactory->expects($this->any())
			->method('get')
			->with('firewall')
			->willReturn(\OC::$server->getL10NFactory()->get('firewall'));

		$this->dispatcher = $this->getMockBuilder('Symfony\Component\EventDispatcher\EventDispatcherInterface')
			->disableOriginalConstructor()
			->getMock();

		$this->instance = $this->getInstance();
		parent::setUp();

	}

	/**
	 * @param array $methods
	 * @param bool $realDispatcher
	 * @return Firewall|\PHPUnit_Framework_MockObject_MockObject
	 */
	protected function getInstance($methods = [], $realDispatcher = false) {
		if ($realDispatcher) {
			$dispatcher = \OC::$server->getEventDispatcher();
		} else {
			$dispatcher = $this->dispatcher;
		}
		if (empty($methods)) {
			return new Firewall(
				$this->config,
				$this->context,
				$this->rulesMock,
				$this->logger,
				$this->userSessionMock,
				$dispatcher,
				$this->groupManager,
				$this->requestMock,
				$this->languageFactory,
				$this->urlGenerator
			);
		} else {
			return $this->getMockBuilder('OCA\Firewall\Firewall')
				->setConstructorArgs([
					$this->config,
					$this->context,
					$this->rulesMock,
					$this->logger,
					$this->userSessionMock,
					$dispatcher,
					$this->groupManager,
					$this->requestMock,
					$this->languageFactory,
					$this->urlGenerator,
				])
				->setMethods($methods)
				->getMock();
		}
	}

}
