<?php
/**
 * ownCloud Firewall
 *
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Firewall\Tests\Lib;


use OCA\Firewall\Ruler;
use OCA\Firewall\Rules;
use Test\TestCase;

class RulesTest extends TestCase {
	/** @var \OCA\Firewall\Context|\PHPUnit_Framework_MockObject_MockObject */
	protected $context;

	/** @var \OCA\Firewall\RuleFactory|\PHPUnit_Framework_MockObject_MockObject */
	protected $ruleFactory;

	protected function setUp() {
		parent::setUp();

		$this->context = $this->getMockBuilder('OCA\Firewall\Context')
			->disableOriginalConstructor()
			->getMock();

		$this->ruleFactory = $this->getMockBuilder('OCA\Firewall\RuleFactory')
			->disableOriginalConstructor()
			->getMock();
	}

	/**
	 * @param array $methods
	 * @return Rules|\PHPUnit_Framework_MockObject_MockObject
	 */
	protected function getRulesInstance(array $methods = []) {
		if (empty($methods)) {
			return new Rules(
				$this->ruleFactory,
				$this->context
			);
		} else {
			return $this->getMockBuilder('OCA\Firewall\Rules')
				->setConstructorArgs([
					$this->ruleFactory,
					$this->context
				])
				->setMethods($methods)
				->getMock();
		}
	}

	public function dataEvalCondition() {
		return [
			[[], null, null, false],
			[['id' => 'invalid'], true, null, false],
			[['id' => 'valid', 'operator' => Ruler::OPERATOR_EQUALS, 'value' => 'ruling'], false, true, true],
			[['id' => 'valid', 'operator' => Ruler::OPERATOR_EQUALS, 'value' => 'ruling'], false, false, false],
		];
	}

	/**
	 * @dataProvider dataEvalCondition
	 *
	 * @param array $rule
	 * @param bool $factoryThrows
	 * @param bool $ruleReturns
	 * @param bool $expected
	 */
	public function testEvalCondition($rule, $factoryThrows, $ruleReturns, $expected) {
		$rules = $this->getRulesInstance();

		if ($factoryThrows === true) {
			$this->ruleFactory->expects($this->once())
				->method('getRuleInstance')
				->with($rule['id'])
				->willThrowException(new \InvalidArgumentException());
		} else if ($factoryThrows === false) {
			$ruleInstance = $this->getMockBuilder('\OCA\Firewall\Rules\Contracts\Rule')
				->disableOriginalConstructor()
				->setMethods([
					'doCheck',
					'validateRuleValue',
					'getValidOperators',
				])
				->getMock();
			$ruleInstance->expects($this->once())
				->method('doCheck')
				->with($rule['operator'], $rule['value'])
				->willReturn($ruleReturns);

			$this->ruleFactory->expects($this->once())
				->method('getRuleInstance')
				->with($rule['id'])
				->willReturn($ruleInstance);
		}

		$this->assertSame($expected, $this->invokePrivate($rules, 'evalCondition', [$rule]));
	}

	public function dataValidateRules() {
		return [
			[null, false],
			[10, false],
			['test', false],
			[[], false],
			[json_encode(null), false],
			[json_encode(10), false],
			[json_encode('test'), false],

			[
				json_encode([]),
				json_encode([]),
			],
			[
				json_encode([
					[]
				]),
				json_encode([]),
			],
			[
				json_encode([
					['name' => 'valid', 'rules' => []],
				]),
				json_encode([
					['name' => 'valid', 'rules' => []],
				]),
			],
			[
				json_encode([
					['name' => ['invalid'], 'rules' => []],
				]),
				false
			],
			[
				json_encode([
					['name' => 'invalid', 'rules' => 'invalid'],
				]),
				false
			],
			[
				json_encode([
					['name' => 'valid', 'rules' => [
						[]
					]],
				]),
				false
			],
		];
	}

	/**
	 * @dataProvider dataValidateRules
	 *
	 * @param mixed $config
	 * @param string|bool $expected
	 */
	public function testValidateRules($config, $expected) {
		$rules = $this->getRulesInstance();

		try {
			$validatedConfig = $rules->validateRules($config);
			$this->assertSame($expected, $validatedConfig);
		} catch (\OutOfBoundsException $e) {
			$this->assertFalse($expected, $e->getMessage());
		}
	}


	public function dataValidateRule() {
		return [
			[null, false],
			[10, false],
			['test', false],
			[[], false],

			[
				[
					'id' => 'test',
					'operator' => 'test',
					'value' => 'test',
				],
			],
			[
				[
					'id' => 'test',
					'operator' => ['test'],
					'value' => 'test',
				],
				false,
			],
			[
				[
					'id' => 'test',
					'operator' => 'invalid operator',
					'value' => 'test',
				],
				false,
			],
			[
				[
					'id' => 'test',
					'operator' => 'test',
					'value' => 'test',
					'rules' => [],
				],
				false,
			],
			[
				[
					'id' => 'invalid rule',
					'operator' => 'test',
					'value' => 'test',
				],
				false,
				false,
			],
		];
	}

	/**
	 * @dataProvider dataValidateRule
	 *
	 * @param mixed $rule
	 * @param bool $isValid
	 * @param bool $isValidRule
	 */
	public function testValidateRule($rule, $isValid = true, $isValidRule = true) {
		$rules = $this->getRulesInstance();

		$ruleMock = $this->getMockBuilder('OCA\Firewall\Rules\Contracts\Rule')
			->disableOriginalConstructor()
			->setMethods([
				'getValidOperators',
				'validateRuleValue',
				'doCheck',
			])
			->getMock();
		$ruleMock->expects($this->any())
			->method('getValidOperators')
			->willReturn(['test']);

		if ($isValidRule) {
			$this->ruleFactory->expects($this->any())
				->method('getRuleInstance')
				->with('test')
				->willReturn($ruleMock);
		} else {
			$this->ruleFactory->expects($this->any())
				->method('getRuleInstance')
				->with('invalid rule')
				->willThrowException(new \InvalidArgumentException());
		}

		try {
			$this->invokePrivate($rules, 'validateRule', [$rule, '']);
			$this->assertTrue($isValid);
		} catch (\OutOfBoundsException $e) {
			$this->assertFalse($isValid, $e->getMessage());
		}
	}

	public function dataCheck() {
		return [
			// Single Rule with two conditions, all don't match
			[
				// Ruleset
				[
					// Single Rule with two conditions
					[
						'name' => 'Rule #1',
						'rules' => [
							[false],
							[false],
						],
					],
				],
				// Number of evaluations
				1,
				// Request should be accepted?
				true,
				// Which rule was offended by the request
				null,
			],
			// Single Rule with two conditions, all first doesn't match
			[
				// Ruleset
				[
					// Single Rule with two conditions
					[
						'name' => 'Rule #1',
						'rules' => [
							[true],
							[false],
						],
					],
				],
				// Number of evaluations
				2,
				// Request should be accepted?
				true,
				// Which rule was offended by the request
				null,
			],
			// Single Rule with two conditions, all match
			[
				// Ruleset
				[
					// Single Rule with two conditions
					[
						'name' => 'Rule #1',
						'rules' => [
							[true],
							[true],
						],
					],
				],
				// Number of evaluations
				2,
				// Request should be accepted?
				false,
				// Which rule was offended by the request
				'Rule #1',
			],

			// Two Rules with one condition, all don't match
			[
				// Ruleset
				[
					[
						'name' => 'Rule #1',
						'rules' => [
							[false],
						],
					],
					[
						'name' => 'Rule #2',
						'rules' => [
							[false],
						],
					],
				],
				// Number of evaluations
				2,
				// Request should be accepted?
				true,
				// Which rule was offended by the request
				null,
			],
			// Two Rules with one condition, all first matches
			[
				// Ruleset
				[
					[
						'name' => 'Rule #1',
						'rules' => [
							[true],
						],
					],
					[
						'name' => 'Rule #2',
						'rules' => [
							[false],
						],
					],
				],
				// Number of evaluations
				1,
				// Request should be accepted?
				false,
				// Which rule was offended by the request
				'Rule #1',
			],
			// Two Rules with one condition, all both match
			[
				// Ruleset
				[
					[
						'name' => 'Rule #1',
						'rules' => [
							[true],
						],
					],
					[
						'name' => 'Rule #2',
						'rules' => [
							[true],
						],
					],
				],
				// Number of evaluations
				1,
				// Request should be accepted?
				false,
				// Which rule was offended by the request
				'Rule #1',
			],
			// Two Rules with one condition, all second matches
			[
				// Ruleset
				[
					[
						'name' => 'Rule #1',
						'rules' => [
							[false],
						],
					],
					[
						'name' => 'Rule #2',
						'rules' => [
							[true],
						],
					],
				],
				// Number of evaluations
				2,
				// Request should be accepted?
				false,
				// Which rule was offended by the request
				'Rule #2',
			],
			// No rules
			[
				// Ruleset
				[],
				// Number of evaluations
				0,
				// Request should be accepted?
				true,
				// Which rule was offended by the request
				null,
			],
			// No rules
			[
				// Ruleset
				[
					[]
				],
				// Number of evaluations
				0,
				// Request should be accepted?
				true,
				// Which rule was offended by the request
				null,
			],
		];
	}

	/**
	 * @dataProvider dataCheck
	 * @param array $rulesSet
	 * @param int $numberOfEvaluations
	 * @param bool $expected
	 * @param string $offender
	 */
	public function testCheck(array $rulesSet, $numberOfEvaluations, $expected, $offender) {
		$rulesInstance = $this->getRulesInstance([
			'evalCondition',
		]);

		$rulesInstance->expects($this->exactly($numberOfEvaluations))
			->method('evalCondition')
			->willReturnCallback(function ($rule) {
				return $rule[0];
			});

		$this->assertSame($expected, $rulesInstance->check($rulesSet));
		$this->assertSame($offender, $rulesInstance->getOffender());
	}
}
