<?php
/**
 * ownCloud Firewall
 *
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Firewall\Tests\Rules;


use OCA\Firewall\Ruler;
use OCA\Firewall\Tests\Helpers\RuleTester;

class CidrTest extends RuleTester {

	public function dataValidateRuleValue() {
		return [
			// IPv4
			['8.8.8.0/-24', true],
			['8.8.8.0/-1', true],
			['8.8.8.0/0', true],
			['8.8.8.0/1', false],
			['8.8.8.0/32', false],
			['8.8.8.0/33', true],
			['8.8.8.0/128', true],
			['8.8.8.0/129', true],

			// IPv6 range
			['2001:db8:abc:1400::/-24', true],
			['2001:db8:abc:1400::/-1', true],
			['2001:db8:abc:1400::/0', true],
			['2001:db8:abc:1400::/1', false],
			['2001:db8:abc:1400::/32', false],
			['2001:db8:abc:1400::/128', false],
			['2001:db8:abc:1400::/129', true],

			['2001:DB8:ABC:1400::/1', false],
			['2001:1400::/1', false],
			['2001:db8:abc:17ff:ffff:ffff:ffff:ffff/1', false],

			// IPv6 ip
			['11111::/32', true],
			['abcd:efg::/32', true],
			['abcd1::/32', true],

			// IPv4 in IPv6
			['::ffff:8.8.8.0/0', true],
			['::ffff:8.8.8.0/1', false],
			['::ffff:8.8.8.0/32', false],
			['::ffff:8.8.8.0/128', false],
			['::ffff:8.8.8.0/129', true],

			['test', true],
			[0, true],
			[42, true],
			[[], true],
			[['test'], true],
			[null, true],
		];
	}

	/**
	 * @return string
	 */
	protected function getRuleClass() {
		return '\OCA\Firewall\Rules\CIDR';
	}

	public function dataDoCheck() {
		return [
			['', '192.168.1.0/24', Ruler::OPERATOR_EQUALS, true, true],
			['', '192.168.1.0/24', Ruler::OPERATOR_EQUALS, false, false],
			['192.168.1.0', '192.168.1.0/24', Ruler::OPERATOR_EQUALS, true, true],
			['192.168.1.0', '192.168.1.0/24', Ruler::OPERATOR_EQUALS, false, false],
			['192.168.1.0', '::192.168.1.0/24', Ruler::OPERATOR_EQUALS, true, true],
			['192.168.1.0', '::192.168.1.0/24', Ruler::OPERATOR_EQUALS, false, false],
			['::ffff:c0a8:100', '192.168.1.0/24', Ruler::OPERATOR_EQUALS, true, true],
			['::ffff:c0a8:100', '192.168.1.0/24', Ruler::OPERATOR_EQUALS, false, false],
			['::ffff:c0a8:100', '::192.168.1.0/24', Ruler::OPERATOR_EQUALS, true, true],
			['::ffff:c0a8:100', '::192.168.1.0/24', Ruler::OPERATOR_EQUALS, false, false],

			['', '192.168.1.0/24', Ruler::OPERATOR_NOT_EQUALS, true, true],
			['', '192.168.1.0/24', Ruler::OPERATOR_NOT_EQUALS, false, false],
			['192.168.1.0', '192.168.1.0/24', Ruler::OPERATOR_NOT_EQUALS, true, true],
			['192.168.1.0', '192.168.1.0/24', Ruler::OPERATOR_NOT_EQUALS, false, false],
			['192.168.1.0', '::192.168.1.0/24', Ruler::OPERATOR_NOT_EQUALS, true, true],
			['192.168.1.0', '::192.168.1.0/24', Ruler::OPERATOR_NOT_EQUALS, false, false],
			['::ffff:c0a8:100', '192.168.1.0/24', Ruler::OPERATOR_NOT_EQUALS, true, true],
			['::ffff:c0a8:100', '192.168.1.0/24', Ruler::OPERATOR_NOT_EQUALS, false, false],
			['::ffff:c0a8:100', '::192.168.1.0/24', Ruler::OPERATOR_NOT_EQUALS, true, true],
			['::ffff:c0a8:100', '::192.168.1.0/24', Ruler::OPERATOR_NOT_EQUALS, false, false],
		];
	}

	/**
	 * @dataProvider dataDoCheck
	 *
	 * @param string $operator
	 * @param string $ip
	 * @param string $ruleValue
	 * @param bool $checkResult
	 * @param bool $expected
	 */
	public function testDoCheck($operator, $ip, $ruleValue, $checkResult, $expected) {
		/** @var \OCA\Firewall\Rules\CIDR|\PHPUnit_Framework_MockObject_MockObject $rule */
		$rule = $this->getMockBuilder('\OCA\Firewall\Rules\CIDR')
			->setConstructorArgs([
				$this->ruler,
				$this->context,
			])
			->setMethods([
				'performCheck',
			])
			->getMock();

		$rule->expects($this->once())
			->method('performCheck')
			->with($operator, $ruleValue, $ip)
			->willReturn($checkResult);

		$this->context->expects($this->once())
			->method('getRemoteAddress')
			->willReturn($ip);

		$this->assertSame($expected, $rule->doCheck($operator, $ruleValue));
	}

	public function dataPerformCheckEqual() {
		return [
			['', '8.8.8.0/24', false, false, false, false],
			['', '::1/24', false, false, false, false],
			['8.8.8.245', '8.8.8.0/24', true, false, true, true],
			['8.8.7.245', '8.8.8.0/24', true, false, false, false],
			['8.8.8.245', '::8.8.8.0/124', false, true, true, true],
			['8.8.7.245', '::8.8.8.0/124', false, true, false, false],
			['::8.8.8.245', '8.8.8.0/24', false, false, true, false],
			['::8.8.7.245', '8.8.8.0/24', false, false, false, false],
			['::8.8.8.245', '::8.8.8.0/124', false, true, true, true],
			['::8.8.7.245', '::8.8.8.0/124', false, true, false, false],
		];
	}

	/**
	 * @dataProvider dataPerformCheckEqual
	 *
	 * @param string $ip
	 * @param string $ruleValue
	 * @param bool $checksIPv4
	 * @param bool $checksIPv6
	 * @param bool $checkResult
	 * @param bool $expected
	 */
	public function testPerformCheckEqual($ip, $ruleValue, $checksIPv4, $checksIPv6, $checkResult, $expected) {
		/** @var \OCA\Firewall\Rules\CIDR|\PHPUnit_Framework_MockObject_MockObject $rule */
		$rule = $this->getMockBuilder('\OCA\Firewall\Rules\CIDR')
			->setConstructorArgs([
				$this->ruler,
				$this->context,
			])
			->setMethods([
				'cidrMatch',
				'cidrMatchIPv6',
			])
			->getMock();

		$rule->expects($checksIPv4 ? $this->once() : $this->never())
			->method('cidrMatch')
			->with($ruleValue, $ip)
			->willReturn($checkResult);
		$rule->expects($checksIPv6 ? $this->once() : $this->never())
			->method('cidrMatchIPv6')
			->with($ruleValue, $ip)
			->willReturn($checkResult);

		$this->assertSame($expected, $this->invokePrivate($rule, 'performCheck', [Ruler::OPERATOR_EQUALS, $ruleValue, $ip]));
	}

	public function dataPerformCheckNotEqual() {
		return [
			['', '8.8.8.0/24', false, false, false, false],
			['', '::1/24', false, false, false, false],
			['8.8.8.245', '8.8.8.0/24', true, false, true, false],
			['8.8.7.245', '8.8.8.0/24', true, false, false, true],
			['8.8.8.245', '::8.8.8.0/24', false, true, true, false],
			['8.8.7.245', '::8.8.8.0/24', false, true, false, true],
			['::8.8.8.245', '8.8.8.0/24', false, false, true, false],
			['::8.8.7.245', '8.8.8.0/24', false, false, false, false],
			['::8.8.8.245', '::8.8.8.0/124', false, true, true, false],
			['::8.8.7.245', '::8.8.8.0/124', false, true, false, true],
		];
	}

	/**
	 * @dataProvider dataPerformCheckNotEqual
	 *
	 * @param string $ip
	 * @param string $ruleValue
	 * @param bool $checksIPv4
	 * @param bool $checksIPv6
	 * @param bool $checkResult
	 * @param bool $expected
	 */
	public function testPerformCheckNotEqual($ip, $ruleValue, $checksIPv4, $checksIPv6, $checkResult, $expected) {
		/** @var \OCA\Firewall\Rules\CIDR|\PHPUnit_Framework_MockObject_MockObject $rule */
		$rule = $this->getMockBuilder('\OCA\Firewall\Rules\CIDR')
			->setConstructorArgs([
				$this->ruler,
				$this->context,
			])
			->setMethods([
				'cidrMatch',
				'cidrMatchIPv6',
			])
			->getMock();

		$rule->expects($checksIPv4 ? $this->once() : $this->never())
			->method('cidrMatch')
			->with($ruleValue, $ip)
			->willReturn($checkResult);
		$rule->expects($checksIPv6 ? $this->once() : $this->never())
			->method('cidrMatchIPv6')
			->with($ruleValue, $ip)
			->willReturn($checkResult);

		$this->assertSame($expected, $this->invokePrivate($rule, 'performCheck', [Ruler::OPERATOR_NOT_EQUALS, $ruleValue, $ip]));
	}

	public function dataCidrMatch() {
		return [
			['8.8.8.245', true],
			['8.8.7.245', false],
		];
	}

	/**
	 * @dataProvider dataCidrMatch
	 *
	 * @param string $ip
	 * @param bool $expected
	 */
	public function testCidrMatch($ip, $expected) {
		$this->assertSame($expected, $this->invokePrivate($this->rule, 'cidrMatch', ['8.8.8.0/24', $ip]));
	}

	public function dataCidrMatchIPv6() {
		return [
			['2001:db8:abc:13ff:ffff:ffff:ffff:ffff', '2001:db8:abc:1400::/54', false],
			['2001:db8:abc:1400:0000:0000:0000:0000', '2001:db8:abc:1400::/54', true],
			['2001:db8:abc:17ff:ffff:ffff:ffff:ffff', '2001:db8:abc:1400::/54', true],
			['2001:db8:abc:1800:0000:0000:0000:0000', '2001:db8:abc:1400::/54', false],

			['2001:db8:abc:0fff:ffff:ffff:ffff:ffff', '2001:db8:abc:1400::/53', false],
			['2001:db8:abc:1000:0000:0000:0000:0000', '2001:db8:abc:1400::/53', true],
			['2001:db8:abc:13ff:ffff:ffff:ffff:ffff', '2001:db8:abc:1400::/53', true],
			['2001:db8:abc:1400:0000:0000:0000:0000', '2001:db8:abc:1400::/53', true],
			['2001:db8:abc:17ff:ffff:ffff:ffff:ffff', '2001:db8:abc:1400::/53', true],
			['2001:db8:abc:1800:0000:0000:0000:0000', '2001:db8:abc:1400::/53', false],

			['2001:db8:abc:0fff:ffff:ffff:ffff:ffff', '2001:db8:abc:1400::/52', false],
			['2001:db8:abc:1000:0000:0000:0000:0000', '2001:db8:abc:1400::/52', true],
			['2001:db8:abc:13ff:ffff:ffff:ffff:ffff', '2001:db8:abc:1400::/52', true],
			['2001:db8:abc:1400:0000:0000:0000:0000', '2001:db8:abc:1400::/52', true],
			['2001:db8:abc:17ff:ffff:ffff:ffff:ffff', '2001:db8:abc:1400::/52', true],
			['2001:db8:abc:1800:0000:0000:0000:0000', '2001:db8:abc:1400::/52', true],
			['2001:db8:abc:18ff:ffff:ffff:ffff:ffff', '2001:db8:abc:1400::/52', true],
			['2001:db8:abc:1fff:ffff:ffff:ffff:ffff', '2001:db8:abc:1400::/52', true],
			['2001:db8:abc:2fff:ffff:ffff:ffff:ffff', '2001:db8:abc:1400::/52', false],

			// Test IPv4 embedding
			['::ffff:127.0.0.1', '::ffff:7f00:1/128', true],
			['::ffff:127.0.0.1', '::ffff:7f00:2/128', false],
			['127.0.0.1', '::ffff:7f00:1/128', true],
			['127.0.0.1', '::ffff:7f00:2/128', false],

			['::ffff:127.0.0.1', '::ffff:7f00:2/124', true],
			['127.0.0.1', '::ffff:7f00:2/124', true],
		];
	}

	/**
	 * @dataProvider dataCidrMatchIPv6
	 * @param string $ip
	 * @param bool $expected
	 */
	public function testDoCheckIPv6($ip, $range, $expected) {
		$this->assertSame($expected, $this->invokePrivate($this->rule, 'cidrMatchIPv6', [$range, $ip]));
	}
}
