<?php
/**
 * ownCloud Firewall
 *
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Firewall\Tests\Rules;


use OCA\Firewall\Context;
use OCA\Firewall\Ruler;
use OCA\Firewall\Rules\DeviceType;
use OCA\Firewall\Tests\Helpers\RuleTester;

class DeviceTypeTest extends RuleTester {

	/** @var \OCA\Firewall\Config|\PHPUnit_Framework_MockObject_MockObject */
	protected $config;

	public function setUp() {
		parent::setUp();

		$this->config = $this->getMockBuilder('OCA\Firewall\Config')
			->disableOriginalConstructor()
			->getMock();

		$this->rule = new DeviceType(
			$this->ruler,
			$this->context,
			$this->config
		);
	}

	public function dataValidateRuleValue() {
		return [
			[Context::SYNC_CLIENT_ANDROID, false],
			[Context::SYNC_CLIENT_IOS, false],
			[Context::SYNC_CLIENT_DESKTOP, false],
			[Context::SYNC_CLIENT_OTHER, false],
			['test', true],
			[0, true],
			[42, true],
			[[], true],
			[['test'], true],
			[null, true],
		];
	}

	/**
	 * @dataProvider dataValidateRuleValue
	 *
	 * @param mixed $value
	 * @param bool $throws
	 */
	public function testValidateRuleValue($value, $throws) {
		$this->config->expects($this->any())
			->method('getBrandedClients')
			->willReturn([]);

		parent::testValidateRuleValue($value, $throws);
	}

	/**
	 * @return string
	 */
	protected function getRuleClass() {
		return null;
	}

	public function testGetValidValues() {
		$this->config->expects($this->once())
			->method('getBrandedClients')
			->willReturn([]);

		parent::testGetValidValues();
	}

	public function dataGetValidValuesBrandedClients() {
		return [
			[[], 4],
			[[Context::SYNC_CLIENT_ANDROID], 4],
			[[Context::SYNC_CLIENT_ANDROID_BRANDED], 7],
			[[Context::SYNC_CLIENT_IOS], 4],
			[[Context::SYNC_CLIENT_IOS_BRANDED], 7],
			[[Context::SYNC_CLIENT_DESKTOP], 4],
			[[Context::SYNC_CLIENT_DESKTOP_BRANDED], 7],
			[[Context::SYNC_CLIENT_ANDROID_BRANDED, Context::SYNC_CLIENT_DESKTOP_BRANDED], 8],
			[[Context::SYNC_CLIENT_ANDROID_BRANDED, Context::SYNC_CLIENT_IOS_BRANDED, Context::SYNC_CLIENT_DESKTOP_BRANDED], 9],
			[[Context::SYNC_CLIENT_ANDROID, Context::SYNC_CLIENT_IOS_BRANDED, Context::SYNC_CLIENT_DESKTOP_BRANDED], 8],
			[[Context::SYNC_CLIENT_ANDROID_BRANDED, Context::SYNC_CLIENT_IOS, Context::SYNC_CLIENT_DESKTOP_BRANDED], 8],
			[[Context::SYNC_CLIENT_ANDROID_BRANDED, Context::SYNC_CLIENT_IOS_BRANDED, Context::SYNC_CLIENT_DESKTOP], 8],
		];
	}

	/**
	 * @dataProvider dataGetValidValuesBrandedClients
	 * @param array $brandedClients
	 * @param int $expected
	 */
	public function testGetValidValuesBrandedClients($brandedClients, $expected) {
		$this->config->expects($this->atLeastOnce())
			->method('getBrandedClients')
			->willReturn($brandedClients);

		$this->assertCount($expected, $this->rule->getValidValues());
	}

	public function dataDoCheck() {
		return [
			[Context::SYNC_CLIENT_DESKTOP, Context::SYNC_CLIENT_DESKTOP, true],
			[Context::SYNC_CLIENT_DESKTOP, Context::SYNC_CLIENT_IOS, false],
			[Context::SYNC_CLIENT_DESKTOP, Context::SYNC_CLIENT_ANDROID, false],
			[Context::SYNC_CLIENT_DESKTOP, Context::SYNC_CLIENT_OTHER, false],
			[Context::SYNC_CLIENT_IOS, Context::SYNC_CLIENT_DESKTOP, false],
			[Context::SYNC_CLIENT_IOS, Context::SYNC_CLIENT_IOS, true],
			[Context::SYNC_CLIENT_IOS, Context::SYNC_CLIENT_ANDROID, false],
			[Context::SYNC_CLIENT_IOS, Context::SYNC_CLIENT_OTHER, false],
			[Context::SYNC_CLIENT_ANDROID, Context::SYNC_CLIENT_DESKTOP, false],
			[Context::SYNC_CLIENT_ANDROID, Context::SYNC_CLIENT_IOS, false],
			[Context::SYNC_CLIENT_ANDROID, Context::SYNC_CLIENT_ANDROID, true],
			[Context::SYNC_CLIENT_ANDROID, Context::SYNC_CLIENT_OTHER, false],
			[Context::SYNC_CLIENT_OTHER, Context::SYNC_CLIENT_DESKTOP, false],
			[Context::SYNC_CLIENT_OTHER, Context::SYNC_CLIENT_IOS, false],
			[Context::SYNC_CLIENT_OTHER, Context::SYNC_CLIENT_ANDROID, false],
			[Context::SYNC_CLIENT_OTHER, Context::SYNC_CLIENT_OTHER, true],

			[Context::SYNC_CLIENT_NON_BRANDED, Context::SYNC_CLIENT_DESKTOP, true],
			[Context::SYNC_CLIENT_NON_BRANDED, Context::SYNC_CLIENT_DESKTOP_BRANDED, false],
			[Context::SYNC_CLIENT_NON_BRANDED, Context::SYNC_CLIENT_IOS, true],
			[Context::SYNC_CLIENT_NON_BRANDED, Context::SYNC_CLIENT_IOS_BRANDED, false],
			[Context::SYNC_CLIENT_NON_BRANDED, Context::SYNC_CLIENT_ANDROID, true],
			[Context::SYNC_CLIENT_NON_BRANDED, Context::SYNC_CLIENT_ANDROID_BRANDED, false],
			[Context::SYNC_CLIENT_NON_BRANDED, Context::SYNC_CLIENT_OTHER, false],

			[Context::SYNC_CLIENT_BRANDED, Context::SYNC_CLIENT_DESKTOP, false],
			[Context::SYNC_CLIENT_BRANDED, Context::SYNC_CLIENT_DESKTOP_BRANDED, true],
			[Context::SYNC_CLIENT_BRANDED, Context::SYNC_CLIENT_IOS, false],
			[Context::SYNC_CLIENT_BRANDED, Context::SYNC_CLIENT_IOS_BRANDED, true],
			[Context::SYNC_CLIENT_BRANDED, Context::SYNC_CLIENT_ANDROID, false],
			[Context::SYNC_CLIENT_BRANDED, Context::SYNC_CLIENT_ANDROID_BRANDED, true],
			[Context::SYNC_CLIENT_BRANDED, Context::SYNC_CLIENT_OTHER, false],
		];
	}

	/**
	 * @dataProvider dataDoCheck
	 *
	 * @param string $rule
	 * @param string $actual
	 * @param bool $expected
	 */
	public function testDoCheckEquals($rule, $actual, $expected) {
		$this->context->expects($this->once())
			->method('getClientDevice')
			->willReturn($actual);

		$this->assertSame($expected, $this->rule->doCheck(Ruler::OPERATOR_EQUALS, $rule));
	}

	/**
	 * @dataProvider dataDoCheck
	 *
	 * @param string $rule
	 * @param string $actual
	 * @param bool $expected
	 */
	public function testDoCheckNotEquals($rule, $actual, $expected) {
		$this->context->expects($this->once())
			->method('getClientDevice')
			->willReturn($actual);

		$this->assertSame(!$expected, $this->rule->doCheck(Ruler::OPERATOR_NOT_EQUALS, $rule));
	}
}
