<?php
/**
 * ownCloud Firewall
 *
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Firewall\Tests\Rules;


use OCA\Firewall\Ruler;
use OCA\Firewall\Tests\Helpers\RuleTester;

class RegexTest extends RuleTester {

	public function dataValidateRuleValue() {
		return [
			['cidr||(.*)', false],
			['subnet||(.*)', false],
			['request-url||(.*)', false],
			['userAgent||(.*)', false],
			['userGroup||(.*)', false],

			// Invalid regex
			['userGroup||.*)', true],
			// Invalid component
			['test||(.*)', true],
			// No split
			['test', true],

			[0, true],
			[42, true],
			[[], true],
			[['test'], true],
			[null, true],
		];
	}

	/**
	 * @return string
	 */
	protected function getRuleClass() {
		return '\OCA\Firewall\Rules\Regex';
	}

	public function dataDoCheck() {
		return [
			['files', true],
			['files$', true],
			['^files$', false],
			['^http\:\/\/localhost\/index\.php\/apps\/files$', true],
			['^files', false],
			['^http\:\/\/localhost\/index\.php\/apps', true],
		];
	}

	/**
	 * @dataProvider dataDoCheck
	 * @param string $regex
	 * @param bool $expected
	 */
	public function testDoCheck($regex, $expected) {
		$this->context->expects($this->any())
			->method('getRequestFullUrl')
			->willReturn('http://localhost/index.php/apps/files');

		$this->assertSame($expected, $this->rule->doCheck(Ruler::OPERATOR_EQUALS, 'request-url||' . $regex));
	}

	/**
	 * @dataProvider dataDoCheck
	 * @param string $regex
	 * @param bool $expected
	 */
	public function testDoCheckNotEquals($regex, $expected) {
		$this->context->expects($this->any())
			->method('getRequestFullUrl')
			->willReturn('http://localhost/index.php/apps/files');

		$this->assertSame(!$expected, $this->rule->doCheck(Ruler::OPERATOR_NOT_EQUALS, 'request-url||' . $regex));
	}

	public function dataDoCheckWithCommaDelimiteredString() {
		return [
			['^admin$', [], false], #0
			['^admin$', ['admin'], true], #1
			['^admin$', ['1admin'], false], #2
			['^admin$', ['admin2'], false], #3
			['^admin$', ['1admin2'], false], #4
			['^admin$', ['test', 'admin'], true], #5
			['^admin$', ['admin', 'test'], true], #6
		];
	}

	/**
	 * @dataProvider dataDoCheckWithCommaDelimiteredString
	 * @param string $groupName
	 * @param null|array $groups
	 * @param bool $expected
	 */
	public function testDoCheckWithCommaDelimiteredString($groupName, $groups, $expected) {
		if ($groups === null) {
			$groups = ['admin', 'test', 'foo', 'bar', 'baz'];
		}
		$this->context->expects($this->any())
			->method('getUserGroups')
			->willReturn($groups);

		$this->assertSame($expected, $this->rule->doCheck(Ruler::OPERATOR_EQUALS, 'userGroup||' . $groupName));
	}

	/**
	 * @dataProvider dataDoCheckWithCommaDelimiteredString
	 * @param string $groupName
	 * @param null|array $groups
	 */
	public function testDoCheckWithCommaDelimiteredStringNoMatch($groupName, $groups) {
		if ($groups === null) {
			$groups = ['admin', 'test', 'foo', 'bar', 'baz'];
		}
		$this->context->expects($this->any())
			->method('getUserGroups')
			->willReturn($groups);

		$this->assertFalse($this->rule->doCheck(Ruler::OPERATOR_EQUALS, 'userGroup||' . $groupName . '2'));
	}

	/**
	 * @dataProvider dataDoCheckWithCommaDelimiteredString
	 * @param string $groupName
	 * @param null|array $groups
	 * @param bool $expected
	 */
	public function testDoCheckNotEqualWithCommaDelimiteredString($groupName, $groups, $expected) {
		if ($groups === null) {
			$groups = ['admin', 'test', 'foo', 'bar', 'baz'];
		}
		$this->context->expects($this->any())
			->method('getUserGroups')
			->willReturn($groups);

		$this->assertSame(!$expected, $this->rule->doCheck(Ruler::OPERATOR_NOT_EQUALS, 'userGroup||' . $groupName));
	}

	/**
	 * @dataProvider dataDoCheckWithCommaDelimiteredString
	 * @param string $groupName
	 * @param null|array $groups
	 */
	public function testDoCheckNotEqualWithCommaDelimiteredStringNoMatch($groupName, $groups) {
		if ($groups === null) {
			$groups = ['admin', 'test', 'foo', 'bar', 'baz'];
		}
		$this->context->expects($this->any())
			->method('getUserGroups')
			->willReturn($groups);

		$this->assertTrue($this->rule->doCheck(Ruler::OPERATOR_NOT_EQUALS, 'userGroup||' . $groupName . '2'));
	}
}
