<?php
/**
 * ownCloud Firewall
 *
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Firewall\Tests\Rules;


use OCA\Firewall\Ruler;
use OCA\Firewall\Tests\Helpers\RuleTester;

class SubnetTest extends RuleTester {

	public function dataValidateRuleValue() {
		return [
			// IPv4
			['8.8.8.0/-24', true],
			['8.8.8.0/-1', true],
			['8.8.8.0/0', true],
			['8.8.8.0/1', false],
			['8.8.8.0/32', false],
			['8.8.8.0/33', true],
			['8.8.8.0/128', true],
			['8.8.8.0/129', true],

			// IPv6 range
			['2001:db8:abc:1400::/-24', true],
			['2001:db8:abc:1400::/-1', true],
			['2001:db8:abc:1400::/0', true],
			['2001:db8:abc:1400::/1', false],
			['2001:db8:abc:1400::/32', false],
			['2001:db8:abc:1400::/128', false],
			['2001:db8:abc:1400::/129', true],

			['2001:DB8:ABC:1400::/1', false],
			['2001:1400::/1', false],
			['2001:db8:abc:17ff:ffff:ffff:ffff:ffff/1', false],

			// IPv6 ip
			['11111::/32', true],
			['abcd:efg::/32', true],
			['abcd1::/32', true],

			// IPv4 in IPv6
			['::ffff:8.8.8.0/0', true],
			['::ffff:8.8.8.0/1', false],
			['::ffff:8.8.8.0/32', false],
			['::ffff:8.8.8.0/128', false],
			['::ffff:8.8.8.0/129', true],

			['test', true],
			[0, true],
			[42, true],
			[[], true],
			[['test'], true],
			[null, true],
		];
	}

	/**
	 * @return string
	 */
	protected function getRuleClass() {
		return '\OCA\Firewall\Rules\Subnet';
	}

	public function dataDoCheck() {
		return [
			['', '192.168.1.0/24', Ruler::OPERATOR_EQUALS, true, true],
			['', '192.168.1.0/24', Ruler::OPERATOR_EQUALS, false, false],
			['192.168.1.0', '192.168.1.0/24', Ruler::OPERATOR_EQUALS, true, true],
			['192.168.1.0', '192.168.1.0/24', Ruler::OPERATOR_EQUALS, false, false],
			['192.168.1.0', '::192.168.1.0/24', Ruler::OPERATOR_EQUALS, true, true],
			['192.168.1.0', '::192.168.1.0/24', Ruler::OPERATOR_EQUALS, false, false],
			['::ffff:c0a8:100', '192.168.1.0/24', Ruler::OPERATOR_EQUALS, true, true],
			['::ffff:c0a8:100', '192.168.1.0/24', Ruler::OPERATOR_EQUALS, false, false],
			['::ffff:c0a8:100', '::192.168.1.0/24', Ruler::OPERATOR_EQUALS, true, true],
			['::ffff:c0a8:100', '::192.168.1.0/24', Ruler::OPERATOR_EQUALS, false, false],

			['', '192.168.1.0/24', Ruler::OPERATOR_NOT_EQUALS, true, true],
			['', '192.168.1.0/24', Ruler::OPERATOR_NOT_EQUALS, false, false],
			['192.168.1.0', '192.168.1.0/24', Ruler::OPERATOR_NOT_EQUALS, true, true],
			['192.168.1.0', '192.168.1.0/24', Ruler::OPERATOR_NOT_EQUALS, false, false],
			['192.168.1.0', '::192.168.1.0/24', Ruler::OPERATOR_NOT_EQUALS, true, true],
			['192.168.1.0', '::192.168.1.0/24', Ruler::OPERATOR_NOT_EQUALS, false, false],
			['::ffff:c0a8:100', '192.168.1.0/24', Ruler::OPERATOR_NOT_EQUALS, true, true],
			['::ffff:c0a8:100', '192.168.1.0/24', Ruler::OPERATOR_NOT_EQUALS, false, false],
			['::ffff:c0a8:100', '::192.168.1.0/24', Ruler::OPERATOR_NOT_EQUALS, true, true],
			['::ffff:c0a8:100', '::192.168.1.0/24', Ruler::OPERATOR_NOT_EQUALS, false, false],
		];
	}

	/**
	 * @dataProvider dataDoCheck
	 *
	 * @param string $operator
	 * @param string $ip
	 * @param string $ruleValue
	 * @param bool $checkResult
	 * @param bool $expected
	 */
	public function testDoCheck($operator, $ip, $ruleValue, $checkResult, $expected) {
		/** @var \OCA\Firewall\Rules\Subnet|\PHPUnit_Framework_MockObject_MockObject $rule */
		$rule = $this->getMockBuilder('\OCA\Firewall\Rules\Subnet')
			->setConstructorArgs([
				$this->ruler,
				$this->context,
			])
			->setMethods([
				'performCheck',
			])
			->getMock();

		$rule->expects($this->once())
			->method('performCheck')
			->with($operator, $ruleValue, $ip)
			->willReturn($checkResult);

		$this->context->expects($this->once())
			->method('getRemoteServerAddress')
			->willReturn($ip);

		$this->assertSame($expected, $rule->doCheck($operator, $ruleValue));
	}
}
