<?php
/**
 * ownCloud
 *
 * @author Lukas Reschke <lukas@owncloud.com>
 * @author Jörn Friedrich Dreyer <jfd@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\User_Shibboleth\Controller;

use OCA\User_Shibboleth\Db\ShibbolethUserMapper;
use OCP\AppFramework\Controller;
use OCP\AppFramework\Http;
use OCP\AppFramework\Http\DataResponse;
use OCP\IConfig;
use OCP\IRequest;
use OCP\Security\ISecureRandom;

/**
 * Class PasswordController is used to handle WebDAV passwords that can be used
 * in combination with sync clients that don't implement the Shib authentication.
 *
 * @package OCA\User_Shibboleth\Controller
 */
class PasswordController extends Controller {

	protected $userId;
	/** @var IConfig */
	protected $config;
	protected $secureRandom;

	/**
	 * @param string $AppName
	 * @param string $UserId
	 * @param IRequest $request
	 * @param IConfig $config
	 * @param ISecureRandom $secureRandom
	 */
	public function __construct($AppName,
								$UserId,
								IRequest $request,
								IConfig $config,
								ISecureRandom $secureRandom) {
		parent::__construct($AppName, $request);

		$this->userId = $UserId;
		$this->config = $config;
		$this->secureRandom = $secureRandom;
	}

	/**
	 * AJAX handler for checking existence of an alternate non-shibboleth password
	 *
	 * @NoAdminRequired
	 * @return DataResponse with the password setup
	 * @throws \OC\DatabaseException
	 */
	public function hasPassword() {
		// FIXME: Make non-static to be able to unit-test this properly
		if (ShibbolethUserMapper::checkIfUserHasWebDAVPassword($this->userId)) {
			$passwordSetup = $this->getWebDAVPasswordConfiguration();
			$result = [
				'desiredLength' => $passwordSetup[0],
				'desiredGroupLength' => $passwordSetup[1],
			];
			return new DataResponse($result);
		} else {
			return new DataResponse(['message' => 'Could not revoke the password'], Http::STATUS_UNPROCESSABLE_ENTITY);
		}
	}

	/**
	 * AJAX handler for generating a new alternate WebDAV password
	 *
	 * @NoAdminRequired
	 * @return DataResponse
	 * @throws \OC\DatabaseException
	 */
	public function getNewPassword() {
		$passwordSetup = $this->getWebDAVPasswordConfiguration();
		$result = [];
		$desiredLength = $passwordSetup[0];
		$desiredGroupLength = $passwordSetup[1];
		$numOfGroups = (int)($desiredLength / $desiredGroupLength);

		while($numOfGroups > 0) {
			$result[] = $this->secureRandom
				->getMediumStrengthGenerator()
				->generate(
					$desiredGroupLength,
					ISecureRandom::CHAR_DIGITS.ISecureRandom::CHAR_LOWER.ISecureRandom::CHAR_UPPER
				);
			$numOfGroups--;
		}

		// FIXME: Make non-static to be able to unit-test this properly
		$fullPassword = implode($result);
		if (ShibbolethUserMapper::setWebDAVPassword($this->userId, $fullPassword)) {
			return new DataResponse($result);
		} else {
			return new DataResponse(['message' => 'Could not save generated password'], Http::STATUS_UNPROCESSABLE_ENTITY);
		}
	}


	/**
	 * AJAX handler for revoking an alternate non-shibboleth password
	 *
	 * @NoAdminRequired
	 * @return DataResponse
	 * @throws \OC\DatabaseException
	 */
	public function revokePassword() {
		// FIXME: Make non-static to be able to unit-test this properly
		if (ShibbolethUserMapper::clearWebDAVPassword($this->userId)) {
			return new DataResponse();
		} else {
			return new DataResponse(['message' => 'Could not revoke the password'], Http::STATUS_UNPROCESSABLE_ENTITY);
		}
	}

	/**
	 * Gets password configuration. Falls back to defaults if no
	 * config is found or it is not sensible (lengths are less
	 * than 1). The defaults are 12 for password length, 4 for
	 * group length.
	 *
	 * @return array Array with two elements, first is a password
	 * length, second is a group length
	 */
	public function getWebDAVPasswordConfiguration() {
		$passwordDefaultLength = 12;
		$groupDefaultLength = 4;

		$passwordLength = $this->config->getSystemValue('non_shib_webdav_password_length', $passwordDefaultLength);
		if ($passwordLength < 1) {
			$passwordLength = $passwordDefaultLength;
		}

		$groupLength = $this->config->getSystemValue('non_shib_webdav_password_group_length', $groupDefaultLength);
		if ($groupLength < 1) {
			$groupLength = $groupDefaultLength;
		}

		return [$passwordLength, $groupLength];
	}

}
