<?php
/**
 * ownCloud
 *
 * @author Jörn Friedrich Dreyer <jfd@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\User_Shibboleth\Mode;

use OCA\User_Shibboleth\Db\ShibbolethUserMapper;
use OCP\IConfig;
use OCP\ILogger;

class AutoProvisioning implements Mode {

	/** @var IConfig */
	private $config;

	/** @var ILogger */
	private $logger;

	public function __construct(IConfig $config, ILogger $logger) {
		$this->config = $config;
		$this->logger = $logger;
	}

	/**
	 * extracts email and displayname from the session and stores them in the
	 * user_shibboleth table for autoprovisioning
	 * @param $uid string uid - eg for shibboleth session the eppn - for AD / ADFS the sAMAccountName
	 * @return bool
	 * @throws \OC\DatabaseException
	 */
	public function isSessionActive($uid) {

		// user's email address
		$emailAddress = $this->getEMailAddress();

		// Display name (given name plus surname)
		$displayName = $this->getDisplay();

		// Check if user already exists
		// FIXME move update-or-insert-if-not-exists to db mapper
		$query = \OC_DB::prepare("SELECT * FROM `*PREFIX*shibboleth_user` WHERE `eppn` = ?");
		$result = $query->execute(array($uid));

		// If user doesn't already exist in the DB, then create it.
		if ($result->fetchAll()) {
			$update = \OCP\DB::prepare(
				"UPDATE `*PREFIX*shibboleth_user` SET `email` = ?, `displayname` = ? WHERE `eppn` =  ?");
			$result = $update->execute(array($emailAddress, $displayName, $uid));

			if(\OCP\DB::isError($result)) {
				$this->logger->error(
					\OCP\DB::getErrorMessage($result),
					['app' => 'user_shibboleth']
				);
			}

			$this->updateEMailAddress($emailAddress, $uid);
		} else {
			$insQuery = \OC_DB::prepare(
				"INSERT INTO `*PREFIX*shibboleth_user` (`eppn`, `email`, `displayname`, `webdav_password`) VALUES (?, ?, ?, NULL)");
			$result = $insQuery->execute(array($uid, $emailAddress, $displayName));

			if(\OCP\DB::isError($result)) {
				$this->logger->error(
					\OCP\DB::getErrorMessage($result),
					['app' => 'user_shibboleth']
				);
			}

			$this->updateEMailAddress($emailAddress, $uid);
		}

		return true;
	}

	/**
	 * @return string|null
	 */
	protected function getEMailAddress() {
		$envSourceEmail = $this->config->getAppValue(
			'user_shibboleth', 'env-source-email', 'eppn'
		);
		return $this->get($envSourceEmail);
	}

	/**
	 * @return string|null
	 */
	protected function getDisplay() {
		$envSourceDisplayName = $this->config->getAppValue(
			'user_shibboleth', 'env-source-displayname', 'eppn'
		);
		return $this->get($envSourceDisplayName);
	}

	/**
	 * @return string|null
	 */
	protected function get($key) {
		return isset($_SERVER[$key]) ? $_SERVER[$key] : null;
	}

	/**
	 * @param string $emailAddress
	 * @param string $eppn
	 */
	private function updateEMailAddress($emailAddress, $eppn) {
		if (!is_null($emailAddress)) {
			$this->config->setUserValue(
				$eppn, 'settings', 'email', $emailAddress);
		}
	}

	/**
	 * Check if backend implements actions
	 * @param array $actions bitwise-or'ed actions
	 * @returns boolean
	 *
	 * Returns the supported actions as int to be
	 * compared with OC_USER_BACKEND_CREATE_USER etc.
	 */
	public function implementsActions($actions) {
		return (bool)((
				\OC_User_Backend::GET_DISPLAYNAME
				| \OC_User_Backend::COUNT_USERS
				| \OC_User_Backend::CHECK_PASSWORD)
			& $actions);
	}

	/**
	 * delete a user
	 * @param string $uid The username of the user to delete
	 * @returns boolean
	 *
	 * Deletes a user
	 */
	public function deleteUser($uid) {
		// FIXME move sql to db mapper to make it mockable
		$connection = \OC::$server->getDatabaseConnection();
		$count = $connection->executeUpdate('DELETE FROM `*PREFIX*shibboleth_user` WHERE LOWER(`eppn`) = LOWER(?)', [$uid]);

		return ($count > 0);
	}


	/**
	 * Get a list of all users
	 *
	 * @param string $search
	 * @param integer|null $limit
	 * @param integer|null $offset
	 * @return string[] with all uids
	 * @throws \OC\DatabaseException
	 */
	public function getUsers($search = '', $limit = null, $offset = null) {
		// FIXME move sql to db mapper to make it mockable
		$query = \OC_DB::prepare('SELECT `eppn` FROM `*PREFIX*shibboleth_user` WHERE LOWER(`eppn`) LIKE LOWER(?)', $limit, $offset);
		$result = $query->execute(array($search.'%'));
		$users = array();
		while ($row = $result->fetchRow()) {
			$users[] = $row['eppn'];
		}
		return $users;
	}

	/**
	 * check if a user exists
	 * @param string $uid the username
	 * @return boolean
	 * @throws \OC\DatabaseException
	 */
	public function userExists($uid) {
		// FIXME move sql to db mapper to make it mockable
		$query = \OC_DB::prepare( 'SELECT * FROM `*PREFIX*shibboleth_user` WHERE LOWER(`eppn`) = LOWER(?)' );
		$result = $query->execute( array( $uid ));
		if ($result->fetchRow()) {
			return true;
		}

		return false;
	}

	/**
	 * get display name of the user
	 * @param string $uid user ID of the user
	 * @return string display name
	 * @throws \OC\DatabaseException
	 */
	public function getDisplayName($uid) {
		// FIXME move sql to db mapper to make it mockable
		$query = \OC_DB::prepare('SELECT * FROM `*PREFIX*shibboleth_user` WHERE LOWER(`eppn`) = LOWER(?)');
		$result = $query->execute( array( $uid ));
		if ($row = $result->fetchRow()) {
			if (!empty($row['displayname'])) {
				return $row['displayname'];
			}
		}

		return $uid;

	}

	/**
	 * Get a list of all display names
	 * @param string $search
	 * @param int|null $limit
	 * @param int|null $offset
	 * @returns array with  all displayNames (value) and the corresponding uids (key)
	 * @throws \OC\DatabaseException
	 *
	 * Get a list of all display names and user ids.
	 */
	public function getDisplayNames($search = '', $limit = null, $offset = null) {
		// FIXME move sql to db mapper to make it mockable
		$displayNames = array();
		$query = \OC_DB::prepare('SELECT `eppn`, `displayname` FROM `*PREFIX*shibboleth_user`'
			.' WHERE LOWER(`eppn`) LIKE LOWER(?) or LOWER(`displayname`) LIKE LOWER(?)', $limit, $offset);
		$result = $query->execute(array('%'.$search.'%', '%'.$search.'%'));
		while ($row = $result->fetchRow()) {
			if (!empty($row['displayname'])) {
				$displayNames[$row['eppn']] = $row['displayname'];
			} else {
				$displayNames[$row['eppn']] = $row['eppn'];
			}
		}

		return $displayNames;
	}

	/**
	 * Check if a user list is available or not
	 * @return boolean if users can be listed or not
	 */
	public function hasUserListings() {
		return true;
	}

	/**
	 * FIXME extract to entity mapper?
	 * counts the users who did log in at least once
	 *
	 * @return int|false
	 * @throws \OC\DatabaseException
	 */
	public function countUsers() {
		// FIXME move sql to db mapper to make it mockable
		$query = \OCP\DB::prepare('SELECT count(*) FROM `*PREFIX*shibboleth_user`');
		$result = $query->execute();
		if (\OCP\DB::isError($result)) {
			return false;
		}
		return $result->fetchOne();
	}

	/**
	 * Check if the password is correct
	 * @param string $uid The username
	 * @param string $password The password
	 * @return string|false
	 * @throws \OC\DatabaseException
	 *
	 * Check if the password is correct without logging in the user
	 * returns the user id or false
	 */
	public function checkPassword($uid, $password) {
		// FIXME move sql to db mapper to make it mockable
		$query = \OC_DB::prepare('SELECT `eppn`, `webdav_password` FROM `*PREFIX*shibboleth_user` WHERE LOWER(`eppn`) = LOWER(?)');
		$result = $query->execute(array($uid));

		$row = $result->fetchRow();
		if ($row) {
			$storedHash = $row['webdav_password'];
			$newHash = '';
			if (\OC::$server->getHasher()->verify($password, $storedHash, $newHash)) {
				if (!empty($newHash)) {
					ShibbolethUserMapper::setWebDAVPassword($uid, $password);
				}
				return $row['eppn'];
			}
		}

		return false;
	}

	/**
	 * Backend name to be shown in user management
	 *
	 * @return string the name of the backend to be shown
	 */
	public function getBackendName() {
		return 'Shibboleth';
	}
}
