<?php
/**
 * ownCloud
 *
 * @author Jörn Friedrich Dreyer <jfd@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\User_Shibboleth\Mode;


use OCP\ILogger;
use OCP\IUserManager;

class SingleSignOn extends \OC_User_Backend implements Mode {

	/** @var ILogger */
	private $logger;

	/** @var IUserManager */
	private $userManager;

	/**
	 * @param ILogger $logger
	 * @param IUserManager $userManager
	 */
	public function __construct(ILogger $logger, IUserManager $userManager) {
		$this->logger = $logger;
		$this->userManager = $userManager;
	}

	/**
	 * checks if the user in the shibboleth session is known by the UserManager
	 * if any other user backend, eg user_ldap or the core database users
	 * implementation knows the uid it will take over and provide displayname,
	 * email etc
	 *
	 * @param $uid string uid - eg for shibboleth session the eppn - for AD / ADFS the sAMAccountName
	 * @return bool
	 */
	public function isSessionActive($uid) {
		// HACK: trigger a search so user_ldap has a chance to cache the user
		//       and return true for the userExists call
		\OC::$server->getUserManager()->search($uid, 1);

		$exists = $this->userManager->userExists($uid);
		if (!$exists) {
			$this->logger->warning(
				"Shibboleth session contains an unknown uid '$uid'",
				['app' => 'user_shibboleth']
			);
		}
		return $exists;
	}

	/**
	 * returns a counts of the users, always false for sso
	 *
	 * @return int|bool
	 */
	public function countUsers() {
		return false;
	}

	/**
	 * Check if the password is correct
	 * @param string $uid The username
	 * @param string $password The password
	 * @return bool
	 *
	 * Check if the password is correct without logging in the user
	 * returns the user id or false
	 */
	public function checkPassword($uid, $password) {
		return false;
	}

	/**
	 * Check if backend implements actions
	 * @param array $actions bitwise-or'ed actions
	 * @returns boolean
	 *
	 * Returns the supported actions as int to be
	 * compared with OC_USER_BACKEND_CREATE_USER etc.
	 */
	public function implementsActions($actions) {
		return false;
	}

	/**
	 * Backend name to be shown in user management
	 *
	 * @return string the name of the backend to be shown
	 */
	public function getBackendName() {
		return 'Shibboleth (SSO)'; // should never make it to the UI
	}
}
