<?php
/**
 * ownCloud
 *
 * @author Thomas Müller <deepdiver@owncloud.com>
 * @author Jörn Friedrich Dreyer <jfd@owncloud.com>
 * @author Krzesimir Nowak
 * @author Iago López Galeiras
 * @copyright (C) 2014-2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\User_Shibboleth;

use OC\HintException;
use OCA\User_Shibboleth\Mode\Mode;
use OCP\Authentication\IApacheBackend;
use OCP\IConfig;
use OCP\ILogger;
use OCP\IUserBackend;
use OCP\UserInterface;


class UserBackend implements UserInterface, IApacheBackend, IUserBackend {

	/** @var IConfig */
	private $config;

	/** @var ILogger */
	private $logger;

	/** @var Mode */
	private $mode;

	public function __construct(IConfig $config, ILogger $logger, Mode $mode) {
		$this->config = $config;
		$this->logger = $logger;
		$this->mode = $mode;
	}

	/**
	* Check if backend implements actions
	* @param array $actions bitwise-or'ed actions
	* @returns boolean
	*
	* Returns the supported actions as int to be
	* compared with OC_USER_BACKEND_CREATE_USER etc.
	*/
	public function implementsActions($actions) {
		return $this->mode->implementsActions($actions);
	}

	/**
	 * returns a counts of the users
	 *
	 * @return int | bool
	 */
	public function countUsers() {
		return $this->mode->countUsers();
	}

	/**
	* delete a user
	* @param string $uid The username of the user to delete
	* @returns boolean
	*
	* Deletes a user
	*/
	public function deleteUser($uid) {
		return $this->mode->deleteUser($uid);
	}

	/**
	 * Get a list of all users
	 *
	 * @param string $search
	 * @param integer|null $limit
	 * @param integer|null $offset
	 * @return string[] with all uids
	 */
	public function getUsers($search = '', $limit = null, $offset = null) {
		return $this->mode->getUsers($search, $limit, $offset);
	}

	/**
	* check if a user exists
	* @param string $uid the username
	* @return boolean
	*/
	public function userExists($uid) {
		return $this->mode->userExists($uid);
	}

	/**
	 * get display name of the user
	 * @param string $uid user ID of the user
	 * @return string display name
	 */
	public function getDisplayName($uid) {
		return $this->mode->getDisplayName($uid);
	}

	/**
	 * Get a list of all display names
	 * @param string $search
	 * @param int|null $limit
	 * @param int|null $offset
	 * @returns array with  all displayNames (value) and the corresponding uids (key)
	 *
	 * Get a list of all display names and user ids.
	 */
	public function getDisplayNames($search = '', $limit = null, $offset = null) {
		return $this->mode->getDisplayNames($search, $limit, $offset);
	}

	/**
	 * Check if a user list is available or not
	 * @return boolean if users can be listed or not
	 */
	public function hasUserListings() {
		return $this->mode->hasUserListings();
	}

	public function isSessionActive() {
		// Is Shibboleth authentication active?
		$mode = $this->config->getAppValue('user_shibboleth', 'mode', UserBackendFactory::MODE_NOT_ACTIVE);
		if ($mode === UserBackendFactory::MODE_NOT_ACTIVE) {
			$this->logger->debug(
				'shibboleth not active', ['app' => 'user_shibboleth']
			);
			return false;
		}

		// we are in a shibboleth session?
		if (!$this->getShibbolethSession()) {
			$this->logger->debug(
				'No Shibboleth session active.', ['app' => 'user_shibboleth']
			);
			return false;
		}

		// we need the auth user to be set
		if (!isset($_SERVER["PHP_AUTH_USER"])) {
			$this->logger->debug(
				'PHP_AUTH_USER not found in _SERVER', ['app' => 'user_shibboleth']
			);
			return false;
		}

		// uid in shibboleth scenarios typically is the eppn / eduPersonPrincipalName (a globally unique user name)
		$uid = $this->getCurrentUserId();
		if (is_null($uid)) {
			$serverInfo = var_export($_SERVER, true);
			$envSourceUid = $this->config->getAppValue(
				'user_shibboleth', 'env-source-uid', 'eppn'
			);
			$this->logger->error(
				"Failed to login because no uid ($envSourceUid) was provided: $serverInfo",
				['app' => 'user_shibboleth']
			);
			throw new HintException("The IdP did not pass the uid ($envSourceUid) properly to this service.",
				'Please contact the administrator to get this resolved.');
		}

		return $this->mode->isSessionActive($uid);
	}

	/**
	 * @return string
	 * JavaScript method.
	 */
	public function getLogoutAttribute() {
		// Using Shibboleth?
		$mode = \OC::$server->getConfig()->getAppValue(
			'user_shibboleth', 'mode', UserBackendFactory::MODE_NOT_ACTIVE
		);
		if ($mode === UserBackendFactory::MODE_NOT_ACTIVE) {
			return '';
		} else if ($this->getShibbolethSession() && \OC_User::isLoggedIn()) {
			return "class='shibboleth-logout'";
		}

		return '';
	}

	/**
	 * Return the Shibboleth session
	 * @return string|null
	 */
	public function getShibbolethSession() {
		$envSourceShibbolethSession = $this->config->getAppValue(
			'user_shibboleth', 'env-source-shib-session', 'Shib-Session-ID'
		);
		return $this->get($envSourceShibbolethSession);
	}

	/**
	 * Return the id of the current user
	 * @return string|null
	 */
	public function getCurrentUserId() {
		$envSourceUid = $this->config->getAppValue(
			'user_shibboleth', 'env-source-uid', 'eppn'
		);
		return $this->get($envSourceUid);
	}

	/**
	 * @return string|null
	 */
	protected function get($key) {
		return isset($_SERVER[$key]) ? $_SERVER[$key] : null;
	}

	/**
	 * FIXME move to utility class
	 */
	public static function setTimezone() {
		if (isset($_POST['timezone-offset'])) {
			if (is_numeric($_POST['timezone-offset'])) {
				\OC::$server->getSession()->set('timezone', intval($_POST['timezone-offset']));
			}
		}
		\OCP\JSON::success();
	}

	/**
	 * Check if the password is correct
	 * @param string $uid The username
	 * @param string $password The password
	 * @return string
	 *
	 * Check if the password is correct without logging in the user
	 * returns the user id or false
	 */
	public function checkPassword($uid, $password) {
		return $this->mode->checkPassword($uid, $password);
	}

	/**
	 * Backend name to be shown in user management
	 *
	 * @return string the name of the backend to be shown
	 */
	public function getBackendName() {
		return $this->mode->getBackendName();
	}

	/**
	 * used for testing
	 * @return Mode
	 */
	public function getMode() {
		return $this->mode;
	}

	/**
	 * FIXME move to utility class
	 * @param array $params
	 */
	public static function setupFS($params) {
		//trigger creation of user home and /files folder
		\OC::$server->getUserFolder($params['user']);
	}
}
