<?php
/**
 * ownCloud
 *
 * @author Jörn Friedrich Dreyer <jfd@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\User_Shibboleth;

use OCA\User_Shibboleth\Mode\AutoProvisioning;
use OCA\User_Shibboleth\Mode\SingleSignOn;
use OCP\IConfig;
use OCP\ILogger;
use OCP\IUserManager;

class UserBackendFactory {

	const MODE_AUTOPROVISION = 'autoprovision';
	const MODE_NOT_ACTIVE = 'notactive';
	const MODE_SSO_ONLY = 'ssoonly';

	/** @var IConfig */
	private $config;

	/** @var ILogger */
	private $logger;

	/** @var IUserManager */
	private $userManager;

	public function __construct(IConfig $config, ILogger $logger, IUserManager $userManager) {
		$this->config = $config;
		$this->logger = $logger;
		$this->userManager = $userManager;
	}

	/**
	 * @param $mode string|null the currently configured mode if it is null the legacy configuration will be checked
	 * @return string
	 */
	public function checkAndUpdateLegacyConfig($mode) {
		if (!$mode) {
			$this->logger->debug(
				"shibboleth mode not configured, checking 'shibboleth_active'",
				['app' => 'user_shibboleth']
			);
			if ($this->config->getAppValue('user_shibboleth', 'shibboleth_active')) {
				$mode = self::MODE_AUTOPROVISION;
				$this->logger->info(
					"converting 'shibboleth_active' configuration to '$mode' mode ",
					['app' => 'user_shibboleth']
				);
				$this->config->deleteAppValue('user_shibboleth', 'shibboleth_active');
				$this->config->setAppValue('user_shibboleth', 'mode', $mode);
			}
		}
		return $mode;
	}

	public function createBackend() {
		$mode = $this->config->getAppValue('user_shibboleth', 'mode', self::MODE_NOT_ACTIVE);
		$mode = $this->checkAndUpdateLegacyConfig($mode);

		if ($mode === self::MODE_SSO_ONLY) {
			$backend = new SingleSignOn($this->logger, $this->userManager);
		} elseif ($mode === self::MODE_AUTOPROVISION) {
			$backend = new AutoProvisioning($this->config, $this->logger);
		} elseif ($mode === self::MODE_NOT_ACTIVE) {
			return null;
		} else {
			// for now fall back to auto provisioning mode
			throw new ConfigurationError(
				"unknown shibboleth mode '$mode' configured"
			);
		}

		return new UserBackend($this->config, $this->logger, $backend);
	}

	/**
	 * @deprecated use DI
	 * @return UserBackend
	 * @throws ConfigurationError
	 */
	public static function createForStaticLegacyCode() {
		$factory = new UserBackendFactory(
			\OC::$server->getConfig(),
			\OC::$server->getLogger(),
			\OC::$server->getUserManager()
		);
		return $factory->createBackend();
	}
}
