<?php
/**
 * ownCloud
 *
 * @author Lukas Reschke <lukas@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\User_Shibboleth\Controller;

use OCA\User_Shibboleth\Db\ShibbolethUserMapper;
use OCP\AppFramework\Http;
use OCP\AppFramework\Http\DataResponse;
use OCP\IConfig;
use Test\TestCase;
use OCP\Security\ISecureRandom;
use OCP\IRequest;

/**
 * Class PasswordControllerTest
 *
 * @package OCA\User_Shibboleth\Tests\Controller
 */
class PasswordControllerTest extends TestCase {
	/** @var IConfig */
	private $config;
	/** @var IRequest */
	private $request;
	/** @var ISecureRandom */
	private $secureRandom;
	/** @var PasswordController */
	private $passwordController;

	public function setUp() {
		parent::setUp();

		$this->request = $this->getMock('\OCP\IRequest');
		$this->config = $this->getMock('OCP\IConfig');
		$this->secureRandom = $this->getMock('\OCP\Security\ISecureRandom');

		$this->passwordController = new PasswordController(
			'user_shibboleth',
			'testUser',
			$this->request,
			$this->config,
			$this->secureRandom
		);

		// FIXME: Hack to create the user for shib due to static code
		$createUserQuery= \OC_DB::prepare(
			'INSERT INTO `*PREFIX*shibboleth_user` (`eppn`, `email`, `displayname`, `webdav_password`) VALUES (?, ?, ?, NULL)'
		);
		$createUserQuery->execute(['testUser', 'test@owncloud.org', 'Just a test user']);
	}

	public function tearDown() {
		// FIXME: Hack to create the user for shib due to static code
		$createUserQuery= \OC_DB::prepare(
			'DELETE FROM`*PREFIX*shibboleth_user` WHERE `eppn` = ?'
		);
		$createUserQuery->execute(['testUser']);
		parent::tearDown();
	}

	public function testHasPassword() {
		$this->config->expects($this->any())
			->method('getSystemValue')
			->will($this->onConsecutiveCalls(12, 4, 12, 4));

		$response = $this->passwordController->hasPassword();
		$this->assertEquals(new DataResponse([
			'message' => 'Could not revoke the password'
		], Http::STATUS_UNPROCESSABLE_ENTITY), $response);

		$result = ShibbolethUserMapper::setWebDAVPassword('testUser', '1234');
		$this->assertTrue((bool)$result);

		$response = $this->passwordController->hasPassword();
		$this->assertEquals(new DataResponse([
			'desiredLength' => 12,
			'desiredGroupLength' => 4
		]), $response);
	}

	public function testGetNewPasswordDefault() {
		$this->config->expects($this->any())
			->method('getSystemValue')
			->will($this->onConsecutiveCalls(12, 4));

		$mediumStrengthGenerator = $this->getMock('\OCP\Security\ISecureRandom');
		$mediumStrengthGenerator
			->expects($this->at(0))
			->method('generate')
			->with(
				4,
				ISecureRandom::CHAR_DIGITS.ISecureRandom::CHAR_LOWER.ISecureRandom::CHAR_UPPER
			)
			->will($this->returnValue('1234'));
		$mediumStrengthGenerator
			->expects($this->at(1))
			->method('generate')
			->with(
				4,
				ISecureRandom::CHAR_DIGITS.ISecureRandom::CHAR_LOWER.ISecureRandom::CHAR_UPPER
			)
			->will($this->returnValue('5678'));
		$mediumStrengthGenerator
			->expects($this->at(2))
			->method('generate')
			->with(
				4,
				ISecureRandom::CHAR_DIGITS.ISecureRandom::CHAR_LOWER.ISecureRandom::CHAR_UPPER
			)
			->will($this->returnValue('9123'));

		$this->secureRandom
			->expects($this->exactly(3))
			->method('getMediumStrengthGenerator')
			->will($this->returnValue($mediumStrengthGenerator));

		$response = $this->passwordController->getNewPassword();
		$expected = new DataResponse(['1234', '5678', '9123']);
		$this->assertEquals($expected, $response);
	}


	public function testGetNewPasswordOdd() {
		$this->config
			->expects($this->any())
			->method('getSystemValue')
			->will($this->onConsecutiveCalls(14, 4));

		$mediumStrengthGenerator = $this->getMock('\OCP\Security\ISecureRandom');
		$mediumStrengthGenerator
			->expects($this->at(0))
			->method('generate')
			->with(
				4,
				ISecureRandom::CHAR_DIGITS.ISecureRandom::CHAR_LOWER.ISecureRandom::CHAR_UPPER
			)
			->will($this->returnValue('1234'));
		$mediumStrengthGenerator
			->expects($this->at(1))
			->method('generate')
			->with(
				4,
				ISecureRandom::CHAR_DIGITS.ISecureRandom::CHAR_LOWER.ISecureRandom::CHAR_UPPER
			)
			->will($this->returnValue('5678'));
		$mediumStrengthGenerator
			->expects($this->at(2))
			->method('generate')
			->with(
				4,
				ISecureRandom::CHAR_DIGITS.ISecureRandom::CHAR_LOWER.ISecureRandom::CHAR_UPPER
			)
			->will($this->returnValue('9123'));

		$this->secureRandom
			->expects($this->exactly(3))
			->method('getMediumStrengthGenerator')
			->will($this->returnValue($mediumStrengthGenerator));

		$response = $this->passwordController->getNewPassword();
		$expected = new DataResponse(['1234', '5678', '9123']);
		$this->assertEquals($expected, $response);
	}

	public function testGetWebDAVPasswordConfiguration() {
		$this->config
			->expects($this->any())
			->method('getSystemValue')
			->will($this->onConsecutiveCalls(16, 2, 0 ,0));

		$result = $this->passwordController->getWebDAVPasswordConfiguration();

		$this->assertEquals(16, $result[0]);
		$this->assertEquals(2, $result[1]);

		$result = $this->passwordController->getWebDAVPasswordConfiguration();

		$this->assertEquals(12, $result[0]);
		$this->assertEquals(4, $result[1]);
	}

}
