<?php
/**
 * ownCloud
 *
 * @author Thomas Müller <deepdiver@owncloud.com>
 * @author Jörn Friedrich Dreyer <jfd@butonic.de>
 * @author Iago López Galeiras
 * @copyright (C) 2014-2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\User_Shibboleth;

use OCA\User_Shibboleth\Db\ShibbolethUserMapper;
use Test\TestCase;

class UserBackendTest extends TestCase {

	/** @var string */
	private $savedMode;

	/** @var string */
	private $savedEnvSourceShibbolethSession;

	/** @var string */
	private $savedEnvSourceUid;

	/**
	 * @var UserBackend
	 */
	private $backend;

	/**
	 * @var string
	 */
	private $testUser;

	/**
	 * @var string
	 */
	private $testUser2;

	/**
	 * @var string
	 */
	private $testUser3;

	/**
	 * @var string
	 */
	private $currentUser;

	/**
	 * @var integer
	 */
	private $savedPasswordLength;

	/**
	 * @var integer
	 */
	private $savedGroupLength;

	/** @var \OCP\IConfig */
	private $config;

	/** @var \OCP\ILogger */
	private $logger;

	/** @var \OCP\IUserManager */
	private $userManager;

	public function setUp() {
		parent::setUp();

		$this->logger = $this->getMockBuilder(
			'\OCP\ILogger')
			->disableOriginalConstructor()
			->getMock();
		$this->userManager = $this->getMockBuilder(
			'\OCP\IUserManager')
			->disableOriginalConstructor()
			->getMock();

		$this->config = \OC::$server->getConfig();

		$this->savedMode = $this->config->getAppValue(
			'user_shibboleth', 'mode', null
		);
		$this->config->setAppValue(
			'user_shibboleth', 'mode', UserBackendFactory::MODE_AUTOPROVISION
		);

		$this->savedEnvSourceShibbolethSession = $this->config->getAppValue(
			'user_shibboleth', 'env-source-shib-session', null
		);
		$this->config->setAppValue(
			'user_shibboleth', 'env-source-shib-session', 'Test-Shib-Session-ID'
		);

		$this->savedEnvSourceUid = $this->config->getAppValue(
			'user_shibboleth', 'env-source-uid', null
		);
		$this->config->setAppValue(
			'user_shibboleth', 'env-source-uid', 'test-eppn'
		);

		$this->currentUser = uniqid('shib-user-') . '@example.com';
		$_SERVER["PHP_AUTH_USER"] = $this->currentUser;
		$_SERVER['test-eppn'] = $this->currentUser;
		$_SERVER['Test-Shib-Session-ID'] = uniqid('shib');

		$factory = new UserBackendFactory(
			$this->config, $this->logger, $this->userManager
		);
		$this->backend = $factory->createBackend();

		$this->testUser = "test_shib_user";
		$this->testUser2 = "test_shib_user2";
		$this->testUser3 = "test_shib_user3";

		$query = \OC_DB::prepare('INSERT INTO `oc_shibboleth_user` VALUES(?, 0, ?, NULL)');
		$query->execute(array($this->testUser, $this->testUser));
		$query->execute(array($this->testUser2, 'displayName'));
		$query->execute(array($this->testUser3, null));

		$this->savedPasswordLength = $this->config->getSystemValue(
			'non_shib_webdav_password_length'
		);
		$this->savedGroupLength = $this->config->getSystemValue(
			'non_shib_webdav_password_group_length'
		);
	}

	public function tearDown() {
		if ($this->savedEnvSourceShibbolethSession) {
			$this->config->setAppValue(
				'user_shibboleth',
				'env-source-shib-session',
				$this->savedEnvSourceShibbolethSession
			);
		} else {
			$this->config->deleteAppValue(
				'user_shibboleth', 'env-source-shib-session'
			);
		}
		if ($this->savedEnvSourceUid) {
			$this->config->setAppValue(
				'user_shibboleth', 'env-source-uid', $this->savedEnvSourceUid
			);
		} else {
			$this->config->deleteAppValue('user_shibboleth', 'env-source-uid');
		}
		if ($this->savedMode) {
			$this->config->setAppValue(
				'user_shibboleth', 'mode', $this->savedMode
			);
		} else {
			$this->config->deleteAppValue('user_shibboleth', 'mode');
		}
		$query = \OC_DB::prepare("DELETE FROM `oc_shibboleth_user` WHERE `eppn` = ?");
		$query->execute(array($this->testUser));
		$query->execute(array($this->testUser2));
		$query->execute(array($this->testUser3));

		if (!is_null($this->savedPasswordLength)) {
			$this->config->setSystemValue(
				'non_shib_webdav_password_length', $this->savedPasswordLength
			);
		} else {
			$this->config->deleteSystemValue('non_shib_webdav_password_length');
		}

		if (!is_null($this->savedGroupLength)) {
			$this->config->setSystemValue(
				'non_shib_webdav_password_group_length', $this->savedGroupLength
			);
		} else {
			$this->config->deleteSystemValue(
				'non_shib_webdav_password_group_length'
			);
		}
		parent::tearDown();
	}

	public function testIsSessionActiveIfNotActive() {
		$this->config->setAppValue('user_shibboleth', 'mode', UserBackendFactory::MODE_NOT_ACTIVE);
		$result = $this->backend->isSessionActive();

		$this->assertFalse($result);

		$result = $this->backend->userExists($this->currentUser);
		$this->assertFalse($result);
	}

	public function testIsSessionActiveNoUserSet() {
		unset($_SERVER["PHP_AUTH_USER"]);
		unset($_SERVER["eppn"]);
		$result = $this->backend->isSessionActive();

		$this->assertFalse($result);

		$result = $this->backend->userExists($this->currentUser);
		$this->assertFalse($result);
	}

	public function testGetCurrentUserId() {
		$result = $this->backend->getCurrentUserId();

		$this->assertEquals($this->currentUser, $result);
	}

	public function testIsSessionActive() {
		$result = $this->backend->isSessionActive();
		$this->assertTrue($result);

		# We call isSessionActive twice to increase code coverage, the first
		# call will create the user if it doesn't exist and the second one just
		# updates its display name and email
		$result = $this->backend->isSessionActive();
		$this->assertTrue($result);

		$result = $this->backend->userExists($this->currentUser);
		$this->assertTrue($result);
	}

	public function testWebDAVPassword() {
		$result = ShibbolethUserMapper::checkIfUserHasWebDAVPassword($this->testUser);
		$this->assertFalse((bool)$result);

		$result = ShibbolethUserMapper::setWebDAVPassword($this->testUser, '1234');
		$this->assertTrue((bool)$result);

		$result = ShibbolethUserMapper::checkIfUserHasWebDAVPassword($this->testUser);
		$this->assertTrue((bool)$result);

		$result = $this->backend->checkPassword($this->testUser, '1234');
		$this->assertTrue((bool)$result);

		$result = $this->backend->checkPassword($this->testUser, '12345');
		$this->assertFalse((bool)$result);

		$result = ShibbolethUserMapper::clearWebDAVPassword($this->testUser);
		$this->assertTrue((bool)$result);

		$result = ShibbolethUserMapper::checkIfUserHasWebDAVPassword($this->testUser);
		$this->assertFalse((bool)$result);
	}

	public function testNonExistentUserWebDAVPasword() {
		$result = $this->backend->userExists('bogusUser');
		$this->assertFalse((bool)$result);

		$result = ShibbolethUserMapper::setWebDAVPassword('bogusUser', '1234');
		$this->assertFalse((bool)$result);

		$result = ShibbolethUserMapper::checkIfUserHasWebDAVPassword('bogusUser');
		$this->assertFalse((bool)$result);

		$result = ShibbolethUserMapper::clearWebDAVPassword('bogusUser');
		$this->assertFalse((bool)$result);

		$result = $this->backend->checkPassword('bogusUser', '12345');
		$this->assertFalse((bool)$result);
	}

	public function testGetDisplayNames() {
		$result = $this->backend->getDisplayNames();
		$this->assertEquals($this->testUser, $result[$this->testUser]);
		$this->assertEquals('displayName', $result[$this->testUser2]);
		$this->assertEquals($this->testUser3, $result[$this->testUser3]);
	}

	public function testSearchDisplayNames() {
		$result = $this->backend->getDisplayNames('name');
		$this->assertArrayHasKey($this->testUser2, $result);
		$this->assertEquals('displayName', $result[$this->testUser2]);
	}

	public function testGetUser() {
		$result = $this->backend->getDisplayName($this->testUser2);
		$this->assertEquals('displayName', $result);
		$result = $this->backend->getDisplayName($this->testUser3);
		$this->assertEquals($this->testUser3, $result);
	}

	public function testGetUsers() {
		$result = $this->backend->getUsers();
		$this->assertTrue(in_array($this->testUser, $result));
		$this->assertTrue(in_array($this->testUser2, $result));
		$this->assertTrue(in_array($this->testUser3, $result));
		$this->assertFalse(in_array('test_shib_user4', $result));
		$userCount = sizeof($result);

		$result = $this->backend->countUsers();
		$this->assertEquals($userCount, $result);

	}

	public function testDelete() {
		$result = $this->backend->deleteUser('unknown');
		$this->assertFalse($result);

		$result = $this->backend->deleteUser('test_shib_user');
		$this->assertTrue($result);

		$result = $this->backend->getUsers();
		$this->assertFalse(in_array($this->testUser, $result));
		$this->assertTrue(in_array($this->testUser2, $result));
		$this->assertTrue(in_array($this->testUser3, $result));

	}
}
